#!/usr/bin/perl 

# Location for Gromacs
my $gromacs = "/usr/local/gromacs/bin";
my $gmx = "gmx";

####Location of the shared files
my $shared = "~/MemProtMD/AT2CG/";

# Location for Memembed
my $membed = "$shared/memembed/bin/memembed";

# Location for DSSP
my $dssp = "$shared/dssp-2.2.1/mkdssp";

my $memembed = "yes" ; # yes or no

my $omp = "no" ; # set to OMP if outer membrane protein

my $netupper = 0.9; # also try 1.0 1.1 1.2?
my $netlower = 0.5;

our %lipidhead = ( 0 => 'NC3', 1 => 'NH3', 2 => 'GLH', 3 => 'B1', 4 => 'CNO', 5 => 'PO1', 6 => 'ROH', 7 => 'PO2', 8 => 'B6', 9 => 'S1', 10 => 'B0', 11 => 'NCO', 12 => 'DOH'); #The idenfity of the atom to use to count the number of lipid particles
our %lipidkey = ( 'DSPC' => 0, 'dspc' => 0, 'DPPC' => 0, 'dppc' => 0, 'DOPC' => 0, 'dopc' => 0,'DHPC' => 0, 'dhpc' => 0, 'DLPC' => 0, 'dlpc' => 0, 'popc' => 0, POPC => 0, 'pope' => 1, 'POPE' => 1, 'pvpe' => 1, 'PVPE' => 1,  'dppe' => 1, 'DPPE' => 1,'dspe' => 1, 'DSPE' => 1, 'dlpe' => 1, 'DLPE' => 1,'POPG' => 2, 'popg' => 2, 'PVPG' => 2, 'pvpg' => 2, 'DSPG' => 2, 'dspg' => 2, 'DPPG' => 2, 'dppg' => 2, 'DLPG' => 2, 'dlpg' => 2, 'DDM' => 3, 'ddm' => 3, 'BOG' => 3, 'bog' => 3, 'POPS' => 4, 'pops' => 4, 'PIP2' => 5, 'pip2' => 5, 'CHOL' => 6, 'chol' => 6, 'CARD' => 7, 'card' => 7, 'DGD' => 8, 'dgd' => 8, 'sqd' => 9, 'SQD' => 9, 'LMG' => 10, 'lmg' => 10, 'POPS' => 11, 'pops' => 11, 'MAG' => 12, 'mag' => 12);

unless ( $ARGV[2] )
{
	die "Usage: <protein-ready.pdb> <number of lipid types to be added> <name of lipid 1> <% of lipid 1> <name of lipid 2> <% of lipids 2>\ne.g. setup.pl 3JYC 1 dppc\ne.g. setup.pl 3JYC 3 card 10 popg 20 pope 70\nN.B. lipid names MUST be be the same as in the directory storing the shared files...\n";
}

my $pdbid = $ARGV[0];
my $lipidtypes = $ARGV[1];

print $pdbid,"\n";

system("rm Protein*.itp");
mkdir "temp-pdb";

if ($lipidtypes == 1){
$ARGV[3] = 100;
}

my @lipidnames;
my @lipidnumbers;

for (my $i = 0; $i < $lipidtypes; $i++)
{
	$lipidnames[$i] = $ARGV[2+(2*$i)];
	$lipidnumbers[$i] = $ARGV[3+(2*$i)];
	$percentage+=$lipidnumbers[$i];
}

unless ($percentage == 100){
	die "\nPercentage of lipids does not equal 100\n"; 
}

system ("grep -v ' O2 ' $pdbid | grep -v 'OXT' | sed 's/ O1 / O  /g' > temp-pdb/no-oxt.pdb");

system ("$gromacs/$gmx pdb2gmx -f temp-pdb/no-oxt.pdb -ignh -ff gromos43a1 -water spc");

system ("$gromacs/$gmx editconf -f conf.gro -o ready-$pdbid -d 1 -resnr 1 -c << EOD
1
EOD");

system ("$gromacs/$gmx editconf -f ready-$pdbid -o temp-pdb/cent.pdb -center 0 0 0 << EOD
0
0
EOD");

if ($memembed eq "yes"){

if (($omp eq "omp")||($omp eq "OMP")){
system ("$membed -o embedded-$pdbid -b -n in temp-pdb/cent.pdb");
}elsif ($omp ne "omp"){
	system ("$membed -o embedded-$pdbid temp-pdb/cent.pdb");
}


system ("$gromacs/$gmx make_ndx -f embedded-$pdbid -o center.ndx << EOD
del 0
del 1-40
r DUM
0|1
q
EOD"); 

system ("$gromacs/$gmx editconf -f embedded-$pdbid -o temp-pdb/center.pdb -c -n center.ndx -d 3 << EOD
0
2
EOD");

system ("grep 'DUM' temp-pdb/center.pdb > temp-pdb/embed.pdb");

}elsif ($memembed eq "no"){
system ("$gromacs/$gmx editconf -f ready-$pdbid -o temp-pdb/center.pdb -c -d 3");
}

$total=0;
open (INP, "temp-pdb/center.pdb") || die "Cannot open center.pdb\n";
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh) = split;
	if ($woa eq ATOM){
	if ($woc eq CA){
		$xhbx[$woe] = $wof;
		$xhby[$woe] = $wog;
		$xhbz[$woe] = $woh;
		$total++;
		}
	}
}
close (INP);

@chain_num=();
$xi=0;
while ($xi<$total)
	{
	$xii=($xi+1);
	$distx=($xhbx[$xi]-$xhbx[$xii])**2;
	$disty=($xhby[$xi]-$xhby[$xii])**2;
	$distz=($xhbz[$xi]-$xhbz[$xii])**2;
	$dist=sqrt($distx+$disty+$distz);
	$dist=$dist/10;
	$dist = sprintf("%.3f", $dist);	
	if ($dist > 0.6){	
	push (@chain_num, $xii);
	}
	$xi++;
}

open SID, "<", "temp-pdb/center.pdb" or die;
open CHA, ">", "temp-pdb/chains.pdb" or die;

$oldres=-999;
$nchain=0;
$chain=();
@refchain = 'A' .. 'Z';
while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,26,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if ($woa eq "ATOM"){
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain];         
             $correct=(1-($chain_num[$nchain]));
             $nchain++;
	     }
	   $res=($woe+$correct);
	   if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } else {print CHA $_,"\n"};
	   $oldres=$res;
           	}
		}
close(CHA);
close(SID);

########  Bondini End ##############

system("python $shared/martinize.py -f temp-pdb/chains.pdb -merge all -dssp $dssp -ff martini22 -v -x protein-cg.pdb -o protein-cg.top -elastic -ef 1000 -el $netlower -eu $netupper -ea 0 -ep 0");

#Copy the required files
my $copyfile = $shared."/mdp_files/em.mdp";
my $command = "cp ".$copyfile." .";
system $command;

$copyfile = $shared."/wat.pdb";
$command = "cp ".$copyfile." ./wat.pdb";
system $command;

$copyfile = $shared."/martini_v2.2.itp";
$command = "cp ".$copyfile." ./martini_v2.2.itp";
system $command;

$copyfile = $shared."/mdp_files/md.mdp";
$command = "cp ".$copyfile." ./md.mdp";
system $command;


#Make the output directory if it does not exist
my $outpath = "MD/";
if (! -d $outpath )
{
        my $command = "mkdir ".$outpath;
        system $command;
}


my $seed = 9999;

open (INB, "temp-pdb/center.pdb") || die "Not possible to open protein-cg.pdb\n";
while (<INB>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woa eq CRYST1){
	$xbox = $wob/10;
	$ybox = $woc/10;
	$zbox = $wod/10;
	}
}

if ($xbox < 12){
	$xbox = 12;
}
if ($ybox < 12){
	$ybox = 12;
}
if ($zbox < 12){
	$zbox = 12;
}

system("mv Protein*.itp protein-cg.itp");

my $prot = "protein.top";
open (OUT, ">$prot" ) || die "Cannot open $temp for writing\n";
printf (OUT "#include \"martini_v2.2.itp\"\n#define RUBBER_BANDS\n#include \"protein-cg.itp\"\n\n[ system ]\nSELF-ASSEMBLY\n\n[ molecules ]\nProtein 1\n");


if ($memembed eq "yes"){
system("cat protein-cg.pdb temp-pdb/embed.pdb | grep -v END > temp-pdb/protein-cg-dum.pdb");

system("$gromacs/$gmx make_ndx -f temp-pdb/protein-cg-dum.pdb -o center-cg.ndx << EOD
del 0
del 1-40
r DUM
q
EOD");

$command = "$gromacs/$gmx editconf -f temp-pdb/protein-cg-dum.pdb -o temp-pdb/cg-temp.pdb -box ".$xbox." ".$ybox." 50 -c -n center-cg.ndx <<EOD
1
0
EOD";
system $command;

$command = "$gromacs/$gmx editconf -f temp-pdb/embed.pdb -o temp-pdb/embed-temp.pdb -box ".$xbox." ".$ybox." 50 -c";
system $command;

$command = "$gromacs/$gmx grompp -f em.mdp -maxwarn 5 -c temp-pdb/cg-temp.pdb -o temp_em -p protein.top";
system $command;

$command = "$gromacs/$gmx mdrun -nt 1 -deffnm temp_em -c temp-pdb/temp_em.pdb";
system $command;

unless (-e "temp-pdb/temp_em.pdb"){
	die "Something went wrong with the Energy Minimisation of the Protein"
}

system("cat temp-pdb/temp_em.pdb temp-pdb/embed-temp.pdb | grep -v END > temp-pdb/protein-cg-dum2.pdb");

$command = "$gromacs/$gmx editconf -f temp-pdb/protein-cg-dum2.pdb -o lipids.pdb -box ".$xbox." ".$ybox." 8 -c -n center-cg.ndx << EOD
1
0
EOD";
system $command;

}

elsif ($memembed eq "no"){
$command = "$gromacs/$gmx editconf -f protein-cg.pdb -o temp-pdb/cg-temp.pdb -box ".$xbox." ".$ybox." 50 -c";
system $command;
$command = "$gromacs/$gmx grompp -f em.mdp -maxwarn 5 -c temp-pdb/cg-temp.pdb -o temp_em -p protein.top";
system $command;

$command = "$gromacs/$gmx mdrun -nt 1 -deffnm temp_em -c temp-pdb/temp_em.pdb";
system $command;
$command = "$gromacs/$gmx editconf -f temp-pdb/temp_em.pdb -o lipids.pdb -box ".$xbox." ".$ybox." 8 -c";
system $command;
}

my $lipidconc = (($xbox*$ybox)*2.5)/100;

print $xbox,"\n";
print $ybox,"\n";
print $lipidconc,"\n";

for (my $i = 0; $i < $lipidtypes; $i++ )
{
my $lipidnum = sprintf("%.0f", (($lipidnumbers[$i])*($lipidconc)));
my $command = "$gromacs/$gmx insert-molecules -f lipids.pdb -rot xyz -nmol ".$lipidnum." -try 1000 -ci ".$shared."/lipids/".$lipidnames[$i].".pdb -o lipids.pdb -radius 0.25";
system $command;
}

#check that nothing has gone wrong with the preparation process - the first residue must be the same in temp.pdb and the original pdb file.

my $originalfile = "protein-cg.pdb";
my $firstresidue;
open (ORIG, $originalfile ) || die "Cannot open $originalfile\n";
while ( my $line = <ORIG> )
{
	next unless ($line =~ m/^ATOM/ );
	$firstresidue = substr($line, 17, 3 );
	last;
}
#print "$firstresidue\n"; 

my $newfile = "lipids.pdb";
my $newfirstresidue;
open (NEW, $newfile ) || die "Cannot open $newfile\n";
while ( my $line = <NEW> )
{
        next unless ($line =~ m/^ATOM/ );
        $newfirstresidue = substr($line, 17, 3 );
        last;
}

unless ( $newfirstresidue eq $firstresidue )
{
	print "Something has gone wrong with lipid addition\n";
	exit;
}

$command = "$gromacs/$gmx editconf -f lipids.pdb -o prot+lipid.pdb -box ".$xbox." ".$ybox." ".$zbox." -c";
system $command;

$command = "$gromacs/$gmx solvate -cp prot+lipid.pdb -o prot+lipid+wat.pdb -cs ".$shared."/wat.pdb -radius 0.25";
system $command;

#### Create the topology file
#### Read the prot+lipid+wat file to work out how many of everything we have
my $file = "prot+lipid+wat.pdb";
my $water = 0;
my @lipids;
for ( my $i = 0; $i < $lipidtypes; $i++ )
{
	$lipids[$i] = 0;
}

open (FILE, $file) || die "Cannot open $file\n";
while (my $line = <FILE> )
{
	next unless ($line =~ m/^ATOM/ );
	my $residue = substr($line, 13, 3);

	for ( my $i = 0; $i < $lipidtypes; $i++ )
	{
		my $key = $lipidkey{$lipidnames[$i]};
		my $match = $lipidhead{$key};
		if ( $residue =~m/$match/ )
		{
			$lipids[$i]++;	
		}
	}
	if ( $residue =~ m/W/ )
	{
		$water++;
	}
}


my $temp = "temp.top";
open (OUT, ">$temp" ) || die "Cannot open $temp for writing\n";
printf (OUT "#include \"martini_v2.2.itp\"\n#include \"protein-cg.itp\"\n\n[ system ]\nSELF-ASSEMBLY\n\n[ molecules ]\nProtein 1\n");
for (my $i = 0; $i < $lipidtypes; $i++ )
{
        my $name = uc ($lipidnames[$i]);
        printf (OUT "%s %d\n", $name, $lipids[$i] );
}
printf (OUT "W %d\n", $water );

$command = "$gromacs/$gmx grompp -maxwarn 5  -f em.mdp -c prot+lipid+wat.pdb -p temp.top -o em.tpr";
system $command;

system("$gromacs/$gmx make_ndx -f prot+lipid+wat.pdb -o SOL_ION.ndx << EOD
del 0-30
aW
q
EOD");

system("$gromacs/$gmx genion -s em.tpr -o prot+lipid+wat+ion.pdb -neutral -conc 0.15 -n SOL_ION.ndx");

my $na = 0;
my $sol = 0;
my $cl = 0;

open (INP, "prot+lipid+wat+ion.pdb") || die "Something went wrong with the ion addition - cannot open prot+lipid+wat+ion.pdb\n";
$atom = 0;
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	if (($woc eq NA)){
	$na++;
	}
	if (($woc eq CL)){
	$cl++;
	}
	if (($woc eq W)){
	$w++;
	}
}

close (INP);
my $topol = "topol.top";
open (OUT, ">$topol" ) || die "Cannot open $temp for writing\n";
printf (OUT "#include \"martini_v2.2.itp\"\n#define RUBBER_BANDS\n\n#include \"protein-cg.itp\"\n\n[ system ]\nSELF-ASSEMBLY\n\n[ molecules ]\nProtein 1\n");
for (my $i = 0; $i < $lipidtypes; $i++ )
{
	my $name = uc ($lipidnames[$i]);
	printf (OUT "%s %d\n", $name, $lipids[$i] );
}

print OUT "\nW $w";
print OUT "\nNA+ $na";
print OUT "\nCL- $cl";
close(OUT);

#Run the energy minimization
$command = "$gromacs/$gmx grompp -maxwarn 5  -f em.mdp -c prot+lipid+wat+ion.pdb -p topol.top -o em.tpr";
system $command;

$command = "$gromacs/$gmx mdrun -nt 1 -v -nice 0 -s em.tpr -deffnm out-em";
system $command;

system("$gromacs/$gmx trjconv -f out-em.gro -o CG-system.pdb -center -pbc res -s em.tpr -conect<<EOD
1
0
EOD");

system("$gromacs/$gmx make_ndx -f CG-system.pdb -o sys.ndx << EOD
del 0
del 1-40
0|rPOP*|rPIP*|rDHP*|rDPP*|rDMP*|rDOP*|rBOG*|rCHO*|rDDM*|rDSP*|rTOC*|rCAR*|rDLP*|rSQD*|rDGD*|rLMG*|rMAG*
1&!0
!1
del 1
name 1 Lipid
name 2 SOL_ION
q
EOD");

$command = "$gromacs/$gmx grompp -maxwarn 5  -f md.mdp -c CG-system.pdb -p topol.top -n sys.ndx -o MD/md.tpr";
system $command;

system ("rm -rf temp* *.list *\#* *.txt ss.xpm scount.xvg posre* mdout.mdp conf.gro");
system ("rm -rf temp* *#* structure.txt *.list *.txt out-em.* charge.* *.log prot+* em.tpr protein.top");
system ("rm -rf MD/*#* tf*.dat sequ.txts split_results *.dat tm.pdb list_protein_names Yoursequenc princ*.pdb topol_Protein* temp-pdb");
open (MD, "MD/md.tpr" ) || die "Something went wrong\n";
close (MD);

print "\n Ready for CG Self-Assembly\n\n";
