#!/usr/bin/perl 
#Revision date: 04/02/2011

#################### Options ############################

# CG Protein itp file 
# Used to centralise the system around the protein.
# Must include parameters for all proteins present.
$itp = "protein-cg.itp";

# Number of individual proteins (multiples of protein-cg.itp)
# Do not change unless your system includes >=2 proteins and you wish to use the alignment mode.
$protnum = 0;

# Lipid deletion method - pymol, genbox, g_membed or none - (required for "align" method) 
$deletion = "none";

# Delete Waters? - genbox, waterkiller or none.
$delete_waters = "genbox";

# Optimise Box? Only works if Bilayer has formed along xy plane.
$optimise = "yes";

############# Do not edit anything below this line ################

############# Locations for Programs etc ################

# The directory containing the atomistic fragments and itp files
$cg2atdir = "/sansom/s46/stansfeld/Convert/CG2AT";

# Base directory for programs
my $machine=`uname -s`;
my $arch=`uname -m`;

if ($machine =~ m/Linux/) {
	if ($arch =~ m/x86_64/) {
$base = "/sbcb/packages/opt/Linux_x86_64/";
	} elsif ($arch =~ m/i686/) {
$base = "/sbcb/packages/opt/Linux_x86/"; 
	}
} 
elsif ($machine =~ m/Darwin/) {
	if ($arch =~ m/x86_64/) {
$base = "/sbcb/packages/opt/Darwin_x86_64/"; 
	} elsif ($arch =~ m/i386/) {
$base = "/sbcb/packages/opt/Darwin_x86/";
	}
}

# Location for Pulchra (including executable)
$pulchra = "$base/pulchra/3.06/bin/pulchra";

# Location for Modeller (including executable)
$modeller = "$base/modeller/9v8/bin/mod9v8";

# Location for Pymol (including executable)
#$pymol = "$base/pymol/1.3/bin/pymol";
$pymol = "/sansom/fedpacks/bin/pymol";

# Location for Gromacs
$gromacs = "$base/gromacs/4.5.3/bin";

########################################################################

$ENV{GMXLIB} = "$gromacs/../share/gromacs/top";

unless ( $ARGV[0] )
{
	die "\nUsage: cg2at.pl CG-system.pdb [original.pdb] \n\nFurther Options within cg2at.pl\n\n";
}

$infile = $ARGV[0];

$method = "full";

if ($ARGV[1]){
$original = $ARGV[1];
$method = "align";
}

print "\n#################################################################\n\n Which Atomistic Forcefield?\n\n";
print "      1 - GROMOS96 43a1 \n";
print "      2 - GROMOS96 43a2 \n";
print "      3 - GROMOS96 45a3 \n";
print "      4 - GROMOS96 53a5 \n";
print "      5 - GROMOS96 53a6 \n";
print "      6 - OPLS United-Atom Lipid (POPC, DOPC, DPPC, DMPC, DHPC) \n";
print "      7 - OPLS All-Atom Lipid (DHPC, DPPC, BOG, DDM) \n";
print "      8 - CHARMM27 \n";
print "      9 - CHARMM36 \n";
print "\n#################################################################\n\n"; 
$contff = <STDIN>;
chop $contff;
if ($contff == "1"){
$ff = "GROMOS96-43a1";
}
elsif ($contff == "2"){
$ff = "GROMOS96-43a2";
}
elsif ($contff == "3"){
$ff = "GROMOS96-45a3";
}
elsif ($contff == "4"){
$ff = "GROMOS96-53a5";
}
elsif ($contff == "5"){
$ff = "GROMOS96-53a6";
}
elsif ($contff == "6"){
$ff = "OPLSUA";
}
elsif ($contff == "7"){
$ff = "OPLSAA";
}
elsif ($contff == "8"){
$ff = "CHARMM27";
}
elsif ($contff == "9"){
$ff = "CHARMM36";
}

`rm -rf CG2AT`;
mkdir('CG2AT');
`cp $cg2atdir/*.itp CG2AT/`;
`cp $itp $infile $original CG2AT/`;
`cp $original CG2AT/cg2atalignfile.pdb`;
chdir "./CG2AT/";

system("$gromacs/editconf -resnr 1 -f $infile -o labelZ.pdb -label Z");

open LABZ, "<", 'labelZ.pdb' or die;
open NOLAB, ">", 'nolabel.pdb' or die;

my @z = <LABZ>;
s/ Z/  /g for @z;
print NOLAB @z;

close (LABZ);
close (NOLAB);

$skip = 0;

$total=0;
$atom = 0;
open (INP, "nolabel.pdb") || die "Cannot open em_inbox.pdb\n";
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh) = split;
	if ($woa eq ATOM){
	if ((($woc eq CA)||($woc =~ /^B/)||($woc =~ /^0B/)||($woc =~ /^4B/)||($woc =~ /^5B/))&&($wod ne DDM)&&($wod ne BOG)){
		$xhbx[$woe] = $wof;
		$xhby[$woe] = $wog;
		$xhbz[$woe] = $woh;
		$total++;
	}
	if ($woc eq CA){
		$atom++;
		}
	}
}
close (INP);

if ($total > 0){
	if ($protnum==0){
		$protnum=1;
	}
open (ITP, "$itp") || ($skip = 1);
close (ITP);
}


if ($skip == 1){
print "\n#################################################################\n\n No $itp file provided.\n\n It's not essential but helps centralise the protein and optimise the box length\n\n Are you sure you wish to continue? (Y/N) ";
$cont = <STDIN>;
chop $cont;
if (($cont =~ "^Y")||($cont =~ "^y")){
print "OK";
}
else {
die "\n Check the location/name of the protein itp file\n\n";
	}
}

if ($method eq "align"){
open (ORIG, "$original") || die "\nCannot open $original as specified by \$original - \nCheck the name and location of your protein structure to be aligned\n\n";
close (ORIG);
}


# bond or martini
if ($atom > 0) {
$forcefield = "bond";
}else{
$forcefield = "martini";
}

$pip2 = 0;
$pip3 = 0;
$popc = 0;
$pope = 0;
$popg = 0;
$pops = 0;
$dopc = 0;
$dppc = 0;
$dmpc = 0;
$dmpe = 0;
$dmpg = 0;
$dhpc = 0;
$bog = 0;
$ddm = 0;
$chol = 0;
$sol = 0;
$na = 0;
$cl = 0;

open (INP, "nolabel.pdb") || die "Cannot open nolabel.pdb\n";
$atom = 0;
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	if (($woc eq PO1)&&($wod eq PIP)){
	$pip2++;
	}
	if (($woc eq PO0)&&($wod eq PI3)){
	$pip3++;
	}
	if (($woc eq NC3)&&($wod eq POP)){
	$popc++;
	}
	if (($woc eq NH3)&&($wod eq POP)){
	$pope++;
	}
	if (($woc eq GLH)&&($wod eq POP)){
	$popg++;
	}
	if (($woc eq NCO)&&($wod eq POP)){
	$pops++;
	}
	if (($woc eq NC3)&&($wod eq DOP)){
	$dopc++;
	}
	if (($woc eq NC3)&&($wod eq DPP)){
	$dppc++;
	}
	if (($woc eq NC3)&&($wod eq DLP)){
	$dmpc++;
	}
	if (($woc eq NH3)&&($wod eq DLP)){
	$dmpe++;
	}
	if (($woc eq NH3)&&($wod eq DSP)){
	$dspe++;
	}
	if (($woc eq GLH)&&($wod eq DLP)){
	$dmpg++;
	}
	if (($woc eq NC3)&&($wod eq DHP)){
	$dhpc++;
	}
	if (($woc eq B1)&&($wod eq DDM)){
	$ddm++;
	}
	if (($woc eq B1)&&($wod eq BOG)){
	$bog++;
	}
	if (($woc eq ROH)&&($wod eq CHO)){
	$chol++;
	}	
	if ($woc eq W){
	$sol++;
	}
	if ($woc =~ m/^NA/){
	$na++;
	}
	if ($woc =~ m/^CL/){
	$cl++;
	}
}
close (INP);

$lipidnum=($pip2+$pip3+$popc+$pope+$popg+$pops+$dopc+$dspe+$dmpc+$dmpe+$dmpg+$dppc+$dhpc+$ddm+$bog+$chol);

open (CGT, "> cg.top") || die "Cannot open cg.top\n";

if ($forcefield eq "bond") {
	print CGT "\n",'#include "ff_v1.4_x.itp"';
	`rm martini_v2.1.itp`;
}else {
	print CGT "\n",'#include "martini_v2.1.itp"';
	`rm ff_v1.4_x.itp`;
}

if ($protnum > 0) {
	print CGT "\n",'#include "',$itp,'"';
}

print CGT "\n",'[ system ]';
print CGT "\n",'[ molecules ]';

if ($protnum > 0) {
print CGT "\nProtein $protnum";
}

print CGT "\nPIP2 $pip2";
print CGT "\nPI3 $pip3";
print CGT "\nPOPC $popc";
print CGT "\nPOPG $popg";
print CGT "\nPOPE $pope";
print CGT "\nPOPS $pops";
print CGT "\nDOPC $dopc";
print CGT "\nDSPE $dspe";
print CGT "\nDPPC $dppc";
print CGT "\nDLPC $dmpc";
print CGT "\nDLPE $dmpe";
print CGT "\nDLPG $dmpg";
print CGT "\nDHPC $dhpc";
print CGT "\nBOG $bog";
if ($forcefield eq "martini"){
print CGT "\nDDM $ddm";
print CGT "\nCHOL $chol";
}
#print CGT "\nW $sol";
#print CGT "\nNA+ $na";
#print CGT "\nCL- $cl";
close (CGT);

open (LIP, "> lipid.top") || die "No Output possible\n";
if ($ff eq "GROMOS96-43a1"){
print LIP "\n",'#include "gromos43a1.ff/forcefield.itp"';
print LIP "\n",'#include "gromos-lipids.itp"';
print LIP "\n",'#include "lipid-berger.itp"';
} elsif ($ff eq "GROMOS96-43a2"){
print LIP "\n",'#include "gromos43a2.ff/forcefield.itp"';
print LIP "\n",'#include "gromos-lipids.itp"';
print LIP "\n",'#include "lipid-berger.itp"';
} elsif ($ff eq "GROMOS96-45a3"){
print LIP "\n",'#include "gromos45a3.ff/forcefield.itp"';
print LIP "\n",'#include "gromos-lipids.itp"';
print LIP "\n",'#include "lipid-berger.itp"';
} elsif ($ff eq "GROMOS96-53a5"){
print LIP "\n",'#include "gromos53a5.ff/forcefield.itp"';
print LIP "\n",'#include "gromos-lipids.itp"';
print LIP "\n",'#include "lipid-berger.itp"';
} elsif ($ff eq "GROMOS96-53a6"){
print LIP "\n",'#include "gromos53a6.ff/forcefield.itp"';
print LIP "\n",'#include "gromos-lipids.itp"';
print LIP "\n",'#include "lipid-berger.itp"';
} elsif ($ff eq "OPLSUA"){
print LIP "\n",'#include "oplsaa.ff/forcefield.itp"';
print LIP "\n",'#include "oplsua-lipids.itp"';
print LIP "\n",'#include "lipid-oplsua.itp"';
} elsif ($ff eq "OPLSAA"){
print LIP "\n",'#include "oplsaa.ff/forcefield.itp"';
print LIP "\n",'#include "oplsaa-lipids.itp"';
print LIP "\n",'#include "lipid-oplsaa.itp"';
} elsif ($ff eq "CHARMM27"){
print LIP "\n",'#include "charmm27.ff/forcefield.itp"';
print LIP "\n",'#include "charmm27-lipids.itp"';
} elsif ($ff eq "CHARMM36"){
print LIP "\n",'#include "charmm36.ff/forcefield.itp"';
print LIP "\n",'#include "charmm36-lipids.itp"';
}
print LIP "\n",'[ system ]';
print LIP "\n",'[ molecules ]';
close (LIP);

if ($protnum > 0) {
	
system("$gromacs/make_ndx -f nolabel.pdb -o protein.ndx << EOD
del 0 
del 1-20
q
EOD");

system("$gromacs/editconf -resnr 1 -f nolabel.pdb -o prot.pdb -n protein.ndx");

open (PROT, "prot.pdb") || die "Cannot open file\n";
open (PRO, "> pro.pdb") || die "No Output possible\n";

while (<PROT>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woa eq ATOM){
	print PRO $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
}
close (PRO);
close (PROT);

}

open (INB, "nolabel.pdb") || die "No Output possible\n";
open (POPC, "> popc.pdb") || die "No Output possible\n";
open (POPE, "> pope.pdb") || die "No Output possible\n";
open (POPG, "> popg.pdb") || die "No Output possible\n";
open (POPS, "> pops.pdb") || die "No Output possible\n";
open (PIP2, "> pip2.pdb") || die "No Output possible\n";
open (PIP3, "> pip3.pdb") || die "No Output possible\n";
open (DHP, "> dhp.pdb") || die "No Output possible\n";
open (DPP, "> dpp.pdb") || die "No Output possible\n";
open (DMPC, "> dmpc.pdb") || die "No Output possible\n";
open (DMPE, "> dmpe.pdb") || die "No Output possible\n";
open (DMPG, "> dmpg.pdb") || die "No Output possible\n";
open (DOP, "> dop.pdb") || die "No Output possible\n";
open (DSPE, "> dspe.pdb") || die "No Output possible\n";
open (DDM, "> ddm.pdb") || die "No Output possible\n";
open (BOG, "> bog.pdb") || die "No Output possible\n";
open (CHOL, "> chol.pdb") || die "No Output possible\n";
#open (WAT, "> wat.pdb") || die "No Output possible\n";
#open (SOD, "> NA.pdb") || die "No Output possible\n";
#open (CHL, "> CL.pdb") || die "No Output possible\n";
open (BOX, "> box.pdb") || die "No Output possible\n";

while (<INB>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woa eq CRYST1){
	print BOX $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	$xbox = $wob/10;
	$ybox = $woc/10;
	$zbox = $wod/10;
	}
	if ($wod eq PIP){
	print PIP2 $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($wod eq PI3){
	print PIP3 $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}	
	if ($wod eq POP){
		if ($woc eq NC3){
			$popcid = $woe;
			print POPC $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popcid){
		print POPC $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	if ($wod eq POP){
		if ($woc eq NH3){
			$popeid = $woe;
			print POPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popeid){
		print POPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	if ($wod eq POP){
		if ($woc eq GLH){
			$popgid = $woe;
			print POPG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popgid){
		print POPG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	if ($wod eq POP){
		if ($woc eq NCO){
			$popsid = $woe;
			print POPS $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popsid){
		print POPS $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	elsif ($wod eq DOP){
	print DOP $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($wod eq DSP){
		if ($woc eq NH3){
			$dspeid = $woe;
			print DSPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dspeid){
		print DSPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	elsif ($wod eq DPP){
	print DPP $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($wod eq DLP){
		if ($woc eq NC3){
			$dlpcid = $woe;
			print DMPC $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dlpcid){
		print DMPC $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	if ($wod eq DLP){
		if ($woc eq NH3){
			$dlpeid = $woe;
			print DMPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dlpeid){
		print DMPE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	if ($wod eq DLP){
		if ($woc eq GLH){
			$dlpgid = $woe;
			print DMPG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dlpgid){
		print DMPG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		}
	}
	elsif ($wod eq DHP){
	print DHP $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	elsif ($wod eq DDM){
	print DDM $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	elsif ($wod eq BOG){
	print BOG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	elsif ($wod eq CHO){
	print CHOL $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
#	elsif ($wod eq W){
#	print WAT $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
#	}
#	elsif ($woc=~ m/^NA/){
#	print SOD $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
#	}
#	elsif ($woc=~ m/^CL/){
#	print CHL $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
#	}
}

close (INB);
close (POPC);
close (POPE);
close (POPG);
close (POPS);
close (PIP2);
close (PIP3);
close (DHP);
close (DPP);
close (DMPC);
close (DMPE);
close (DMPG);
close (DOP);
close (DSPE);
close (DDM);
close (BOG);
close (CHOL);
#close (WAT);
#close (SOD);
#close (CHL);
close (BOX);

`cat box.pdb pro.pdb pip2.pdb pip3.pdb popc.pdb popg.pdb pope.pdb pops.pdb dop.pdb dspe.pdb dpp.pdb dmpc.pdb dmpe.pdb dmpg.pdb dhp.pdb  bog.pdb ddm.pdb chol.pdb > order.pdb`;

open (EM, "> em.mdp") || die "Cannot open em.mdp\n";
print EM "cpp                      = /lib/cpp";
print EM "\nintegrator               = steep" ;
print EM "\nemstep                   = 0.001";
print EM "\nemtol                    = 500";
print EM "\nepsilon_r                = 65" ;
print EM "\nnsteps                   = 5000";
close (EM);

open (EM, "> em-bfgs.mdp") || die "Cannot open em-bfgs.mdp\n";
print EM "cpp                      = /lib/cpp";
print EM "\nintegrator               = l-bfgs" ;
print EM "\nemstep                   = 0.001";
print EM "\nemtol                    = 1000";
print EM "\nepsilon_r                = 65" ;
print EM "\nnsteps                   = 5000";
close (EM);

open (EM, "> em-cg.mdp") || die "Cannot open em-bfgs.mdp\n";
print EM "cpp                      = /lib/cpp";
print EM "\nintegrator               = cg" ;
print EM "\nemstep                   = 0.001";
print EM "\nemtol                    = 1000";
print EM "\nepsilon_r                = 65" ;
print EM "\nnsteps                   = 5000";
close (EM);

system("$gromacs/make_ndx -f order.pdb -o order.ndx  << EOD
del 2-30
1|rPOP|rPIP|rDHP|rDPP|rDMP|rDOP|rBOG|rCHO|rDDM|rDSP
!1|rPOP|rPIP|rDHP|rDPP|rDMP|rDOP|rBOG|rCHO|rDDM|rDSP
q
EOD");

if ($skip == 0){

system("$gromacs/editconf -resnr 1 -f order.pdb -c -o order-center1.pdb -n order.ndx -box $xbox $ybox 30 << EOD
1
0
EOD");

system("$gromacs/grompp -maxwarn 10 -f em.mdp -c order-center1.pdb -o em_inbox1.tpr -p cg.top");

open (ORD, "em_inbox1.tpr") || die "\n########################################################################\n\n Cannot set-up system for centralisation - Check your $itp file\n\n";

system("$gromacs/trjconv -f order-center1.pdb -s em_inbox1.tpr -center -pbc mol -o em_inbox1.pdb<< EOD
1
0
EOD");

if ($optimise eq "yes"){

system("$gromacs/g_traj -ox -f em_inbox1.pdb -s em_inbox1.pdb -n order.ndx -com <<EOD
2
EOD");

$com=0;

open(COM, "coord.xvg");
	while (<COM>) {
	chomp;
	($xoa, $xob, $xoc, $xod) = split;
	if (($xoa ne '#')||($xoa ne '@')){
	$com=$xod;
		}
	}
close (COM);

$top = ($com*10);
$bot = ($com*10);

open (TMP, "em_inbox1.pdb");
	while (<TMP>) {
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
		if (($woa eq "ATOM")&&(($woc eq "CA")||($woc =~ "^B")||($woc eq "C1A"))){
			if ($woh > $top){
			$top = $woh;
			}
			elsif ($woh < $bot){
			$bot = $woh;
			}
		}
	}
	
$topdist = ((sqrt($top*$top))/10); 
$botdist = ((sqrt($bot*$bot))/10); 
$zaxis = (($topdist-$botdist)+5);

sub round {
	my($zaxis) = shift;
	return int($zaxis + 0.5 * ($zaxis <=> 0));
}

$bigz = round($zaxis);

if ($bigz < $zbox){
	$zbox = $bigz;
	}
}

system("$gromacs/editconf -resnr 1 -f em_inbox1.pdb -o order-center.pdb -box $xbox $ybox $zbox -c -n order.ndx << EOD
2
0
EOD");

system("$gromacs/grompp -maxwarn 10 -f em.mdp -c order-center.pdb -o em_inbox -p cg.top");
system("$gromacs/mdrun_d -deffnm em_inbox");
system("$gromacs/editconf -resnr 1 -f em_inbox.gro -o em_inbox.pdb");

}

elsif ($skip == 1) {
system("$gromacs/editconf -resnr 1 -f order.pdb -o em_inbox.pdb -box $xbox $ybox $zbox");
}

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (BOX, "> box.pdb") || die "No Output possible\n";
while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woa eq CRYST1){
	print BOX $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
close (EMI);
close (BOX);

$total=0;
$atom = 0;
open (INP, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh) = split;
	if ($woa eq ATOM){
	if ((($woc eq CA)||($woc =~ /^B/)||($woc =~ /^0B/)||($woc =~ /^4B/)||($woc =~ /^5B/))&&($wod ne DDM)&&($wod ne BOG)){
		$xhbx[$woe] = $wof;
		$xhby[$woe] = $wog;
		$xhbz[$woe] = $woh;
		$total++;
	}
	if ($woc eq CA){
		$atom++;
		}
	}
}
close (INP);

@chain_num=();
$xi=0;
while ($xi<$total)
	{
	$xii=($xi+1);
	$distx=($xhbx[$xi]-$xhbx[$xii])**2;
	$disty=($xhby[$xi]-$xhby[$xii])**2;
	$distz=($xhbz[$xi]-$xhbz[$xii])**2;
	$dist=sqrt($distx+$disty+$distz);
	$dist=$dist/10;
	$dist = sprintf("%.3f", $dist);	
	if ($dist > 0.6){	
	push (@chain_num, $xii);
	}
	$xi++;
}

$chain = @chain_num;

`cp $infile CG-initial-system.pdb`;
`cp em_inbox.pdb CG-centered.pdb`;

#@ARGV = ('NA.pdb', 'CL.pdb');
#open ION, '>', 'ion_atoms.pdb' or die $!;
#while (<>) {
#  print ION;
#}
#close(ION);
#@ARGV = ();

`cp box.pdb bilayer-box.pdb`;
`cp lipid.top popg.top`;
`cp lipid.top pops.top`;

#################### PIP2 ########################

if ($pip2 > 0) {

`cp -R $cg2atdir/PIP2 .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (PIP2, "> pip2-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq PIP){
		if ($woc eq PO1){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"PBN",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO2){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"PBH",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO3){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"PAW",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP1){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"C5 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP2){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"C3 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP3){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"C1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"OBT",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"OAR",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CBY",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CCC",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CCG",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CCK",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CCO",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CAM",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CAI",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CAE",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print PIP2 $spa,$woa,$spb,$wob,$spc,"CAA",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
	}
}
close(EMI);
close(PIP2);

# Directory to store all the best-fitting AT lipids, for concatenation into AT bilayer structure
`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f pip2-cg.pdb -o pip2-cg.pdb");

$num = 1;

while ($num <= $pip2){

$loop = sprintf("%04d", $num);

open (ATM, "pip2-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPIP2 $pip2";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $pip2){

$loop_main = sprintf("%04d", $num_main);

mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/RP.pdb -name -o fit_${loop_main}/fit_RP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP2/C4B.pdb -name -o fit_${loop_main}/fit_C4B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

open (CCP, "fit_CCP.pdb") || die "Cannot open file\n";
open (RP, "fit_RP.pdb") || die "Cannot open file\n";
open (RP2, "fit_RP.pdb") || die "Cannot open file\n";
open (RP3, "fit_RP.pdb") || die "Cannot open file\n";
open (LOOP1, "../Res_CG/res_cg_${loop_main}.pdb") || die "Cannot open file\n";
open (LOOP2, "../Res_CG/res_cg_${loop_main}.pdb") || die "Cannot open file\n";
open (O1, "> O1.pdb") || die "No Output possible\n";
open (O2, "> O2.pdb") || die "No Output possible\n";
open (O3, "> O3.pdb") || die "No Output possible\n";

while (<CCP>){
	if ($_ =~ m/OAV/){ 
		print O1 $_ ;
		}
	if ($_ =~ m/PAW/){ 
		print O1 $_ ;
		}
	}

while (<RP>){
	if ($_ =~ m/OAZ/){ 
		print O1 $_ ;
		}
	if ($_ =~ m/OBM/){ 
		print O1 $_ ;
		}
	}


while (<LOOP1>){
	if ($_ =~ m/PBN/){ 
		print O2 $_ ;
		}
	}

while (<RP2>){
	if ($_ =~ m/OBM/){ 
		print O2 $_ ;
		}
	if ($_ =~ m/OAZ/){ 
		print O2 $_ ;
		}
	if ($_ =~ m/OBG/){ 
		print O3 $_ ;
		}
	}

while (<LOOP2>){
	if ($_ =~ m/PBH/){ 
		print O3 $_ ;
		}
	}

while (<RP3>){
	if ($_ =~ m/OAZ/){ 
		print O3 $_ ;
		}
	}

close (CCP);
close(RP);
close(RP2);
close(RP3);
close (LOOP1);
close (LOOP2);	
close (O1);
close (O2); 
close (O3);

system("$gromacs/g_confrms -nice 0 -one -f1 O1.pdb -f2 ../PIP2/PO1.pdb -name -o fit_PO1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 O2.pdb -f2 ../PIP2/PO2.pdb -name -o fit_PO2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 O3.pdb -f2 ../PIP2/PO3.pdb -name -o fit_PO3.pdb << EOD
1
1
EOD");

system("less fit_C4B.pdb | grep 'ATOM' | grep -v 'CCK' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'ATOM' | grep -v 'CCG'  >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' | grep -v 'CCC'  >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'CBY'  >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'CBY'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBX'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBW'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBU'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'OBV'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'OBT'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAT'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAU'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'OAV'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'PAW'  >> complete_${loop_main}.pdb");
system("less fit_PO1.pdb | grep 'OAX'  >> complete_${loop_main}.pdb");
system("less fit_PO1.pdb | grep 'OAY'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OAZ'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C1 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C6 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBS'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'HBS'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C5 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBM'  >> complete_${loop_main}.pdb");
system("less fit_PO2.pdb | grep 'ATOM' | grep -v 'OBM'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C4 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBG'  >> complete_${loop_main}.pdb");
system("less fit_PO3.pdb | grep 'ATOM' | grep -v 'OBG'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C3 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBE'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'HBE'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C2 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBC'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'HBC'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAS'  >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM'  >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAL' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAK' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAJ' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'CAE' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

`cat fit_*/complete_*.pdb | grep 'ATOM' > pip2-bilayer.pdb`;
`cat pip2-bilayer.pdb >> bilayer-box.pdb`;

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ PIP2/`;
}


#################### PIP3 ########################

if ($pip3 > 0) {

`cp -R $cg2atdir/PIP3 .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (PIP3, "> pip3-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq PI3){
		if ($woc eq PO1){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"PBN",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO2){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"PBH",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO3){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"PAW",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO0){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"PBE",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP1){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"C5 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP2){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"C3 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq RP3){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"C1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"OBT",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"OAR",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CBY",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CCC",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CCG",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CCK",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CCO",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CAM",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CAI",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CAE",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print PIP3 $spa,$woa,$spb,$wob,$spc,"CAA",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
	}
}
close(EMI);
close(PIP3);

# Directory to store all the best-fitting AT lipids, for concatenation into AT bilayer structure
`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f pip3-cg.pdb -o pip3-cg.pdb");

$num = 1;

while ($num <= $pip3){

$loop = sprintf("%04d", $num);

open (ATM, "pip3-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPIP3 $pip3";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $pip3){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid

mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/RP.pdb -name -o fit_${loop_main}/fit_RP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 PIP3/C4B.pdb -name -o fit_${loop_main}/fit_C4B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_CCP.pdb | grep 'OAV' > O1.pdb");
system("less fit_CCP.pdb | grep 'PAW' >> O1.pdb");
system("less fit_RP.pdb | grep 'OAZ' >> O1.pdb");
system("less fit_RP.pdb | grep 'OBM' >> O1.pdb");

system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'PBN' > O2.pdb");
system("less fit_RP.pdb | grep 'OBM' >> O2.pdb");
system("less fit_RP.pdb | grep 'OAZ' >> O2.pdb");

system("less fit_RP.pdb | grep 'OBG' > O3.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'PBH' >> O3.pdb");
system("less fit_RP.pdb | grep 'OAZ' >> O3.pdb");

system("less fit_RP.pdb | grep 'OBE' > O4.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'PBE' >> O4.pdb");
system("less fit_RP.pdb | grep 'OAZ' >> O4.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 O1.pdb -f2 ../PIP3/PO1.pdb -name -o fit_PO1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 O2.pdb -f2 ../PIP3/PO2.pdb -name -o fit_PO2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 O3.pdb -f2 ../PIP3/PO3.pdb -name -o fit_PO3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 O4.pdb -f2 ../PIP3/PO0.pdb -name -o fit_PO0.pdb << EOD
1
1
EOD");

system("less fit_C4B.pdb | grep 'ATOM' | grep -v 'CCK' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'ATOM' | grep -v 'CCG'  >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' | grep -v 'CCC'  >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'CBY'  >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'CBY'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBX'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBW'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'CBU'  >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'OBV'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'OBT'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAT'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAU'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'OAV'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'PAW'  >> complete_${loop_main}.pdb");
system("less fit_PO1.pdb | grep 'OAX'  >> complete_${loop_main}.pdb");
system("less fit_PO1.pdb | grep 'OAY'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OAZ'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C1 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C6 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBS'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'HBS'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C5 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBM'  >> complete_${loop_main}.pdb");
system("less fit_PO2.pdb | grep 'ATOM' | grep -v 'OBM'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C4 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBG'  >> complete_${loop_main}.pdb");
system("less fit_PO3.pdb | grep 'ATOM' | grep -v 'OBG'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C3 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBE'  >> complete_${loop_main}.pdb");
system("less fit_PO0.pdb | grep 'PBE'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'C2 '  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'OBC'  >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'HBC'  >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'CAS'  >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM'  >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAL' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAK' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'CAJ' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'CAE' >> complete_${loop_main}.pdb");
system("less fit_PO0.pdb | grep 'OCA'  >> complete_${loop_main}.pdb");
system("less fit_PO0.pdb | grep 'OCB'  >> complete_${loop_main}.pdb");
system("less fit_PO0.pdb | grep 'OCC'  >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > pip3-bilayer.pdb");
system("cat pip3-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ PIP3/`;
}


#################### POPC ########################

if (($popc > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/POPC .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPC, "> popc-cg.pdb") || die "No output possible\n";

$popcid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NC3){
			$popcid = $woe;	
			print POPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popcid){
		if ($woc eq PO4){	
			print POPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"CA1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C49",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f popc-cg.pdb -o popc-cg.pdb");

$num = 1;

while ($num <= $popc){

$loop = sprintf("%04d", $num);

open (ATM, "popc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPC $popc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $popc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPC/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C50' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA1' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA2' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > popc-bilayer.pdb");
system("cat popc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPC-oplsua/`;
}

#################### POPC OPLSUA ########################

if (($popc > 0)&&($ff eq "OPLSUA")) {

`cp -R $cg2atdir/POPC-oplsua .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPC, "> popc-cg.pdb") || die "No output possible\n";

$popcid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NC3){
			$popcid = $woe;	
			print POPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popcid){
		if ($woc eq PO4){	
			print POPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O35",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C32",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C40",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C44",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C48",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C51",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f popc-cg.pdb -o popc-cg.pdb");

$num = 1;

while ($num <= $popc){

$loop = sprintf("%04d", $num);

open (ATM, "popc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPC $popc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $popc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-oplsua/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPC-oplsua/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C33' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C34' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C42' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C43' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C50' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C51' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C52' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > popc-bilayer.pdb");
system("cat popc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPC-oplsua/`;
}

#################### POPC CHARMM ########################

if (($popc > 0)&&($ff =~ m/^CHARMM/)) {

`cp -R $cg2atdir/POPC-CHARMM .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPC, "> popc-cg.pdb") || die "No output possible\n";

$popcid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NC3){
			$popcid = $woe;	
			print POPC $spa,$woa,$spb,$wob,$spc,"N  ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popcid){
		if ($woc eq PO4){	
			print POPC $spa,$woa,$spb,$wob,$spc,"P1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O21",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPC $spa,$woa,$spb,$wob,$spc,"O31",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C24",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C28",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C2A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C2B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C2C",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C34",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C3A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPC $spa,$woa,$spb,$wob,$spc,"C3B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f popc-cg.pdb -o popc-cg.pdb");

$num = 1;

while ($num <= $popc){

$loop = sprintf("%04d", $num);

open (ATM, "popc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPC $popc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $popc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPC-CHARMM/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'P1' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'O1' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O2' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPC-CHARMM/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep ATOM | grep -v 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O3 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O4 ' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep ATOM | grep 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'C3 ' | grep -v 'HX '| grep -v 'HY '| grep -v 'O31' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'O21'|grep -v 'C23' | grep -v 'H3R'| grep -v 'H3S'| grep -v 'C24'| grep -v 'H4R'| grep -v 'H4S' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'O2 ' | grep -v 'C1 '| grep -v 'HA '| grep -v 'HB ' | grep -v 'C2 '| grep -v 'HS '| grep -v 'O21'>> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C33' | grep -v 'O31'| grep -v 'H3X'| grep -v 'H3Y'| grep -v 'C34'| grep -v 'H4X'| grep -v 'H4Y' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'C21' | grep -v 'O21'| grep -v 'O22'| grep -v 'C22'| grep -v 'H2R'| grep -v 'H2S' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'C24' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' | grep -v 'C28' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'C2A' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'ATOM' | grep -v 'C2B' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C31' | grep -v 'O31'| grep -v 'O32'| grep -v 'C32'| grep -v 'H2X'| grep -v 'H2Y' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'C34' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' | grep -v 'C38' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'ATOM' | grep -v 'C3A' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > popc-bilayer.pdb");
system("cat popc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPC-CHARMM/`;
}

#################### POPG ########################

if (($popg > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/POPG .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPG, "> popg-cg.pdb") || die "No output possible\n";

$popgid = 0;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq GLH){
			$popgid = $woe;	
			print POPG $spa,$woa,$spb,$wob,$spc,"C2 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popgid){
		if ($woc eq PO4){	
			print POPG $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPG $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPG $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"CA1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C49",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPG);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f popg-cg.pdb -o popg-cg.pdb");

$num = 1;

while ($num <= $popg){

$loop = sprintf("%04d", $num);

open (ATM, "popg-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPG $popg";
close (LIP);

open (POPG, ">> popg.top") || die "No output possible\n";
print POPG "\nPOPG $popg";
close (POPG);

# For every CG lipid
$num_main = 1;
while ($num_main <= $popg){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/GLH.pdb -name -o fit_${loop_main}/fit_GLH.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

`less fit_GLH.pdb | grep 'O7' > headgroup_chemistry.pdb`;
`less fit_GLH.pdb | grep 'P8' >> headgroup_chemistry.pdb`;
`less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb`;

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPG/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

`less fit_GLH.pdb | grep 'H0' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'O1' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C2 ' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'C3' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'O4' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'H5' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'C6' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'O7' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'P8' >> complete_${loop_main}.pdb`;
`less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb`;
`less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'O14' >> complete_${loop_main}.pdb`;
`less fit_GL1.pdb | grep 'ATOM' | grep -v 'O14'| grep -v 'C19' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C19' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C23' >> complete_${loop_main}.pdb`;
`less fit_C2A.pdb | grep 'ATOM' | grep -v C23 | grep -v C26 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C26' >> complete_${loop_main}.pdb`;
`less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb`;
`less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'O33' >> complete_${loop_main}.pdb`;
`less fit_GL2.pdb | grep 'ATOM' | grep -v O33 | grep -v C38 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C38' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C42' >> complete_${loop_main}.pdb`;
`less fit_C2B.pdb | grep 'ATOM' | grep -v C42 | grep -v C46 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C46' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C48' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C49' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C50' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'CA1' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'CA2' >> complete_${loop_main}.pdb`;

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > popg-bilayer.pdb");
system("cat box.pdb popg-bilayer.pdb > popg-box.pdb");
system("$gromacs/grompp -f em-bfgs.mdp -c popg-box.pdb -p popg.top -o popg_em -maxwarn 5"); 
system("$gromacs/mdrun -deffnm popg_em");
system("$gromacs/editconf -resnr 1 -f popg_em.gro -o popg_em.pdb");
system("cat popg_em.pdb | grep ATOM >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPG/`;
}

#################### POPG CHARMM ########################

if (($popg > 0)&&($ff =~ m/^CHARMM/)) {

`cp -R $cg2atdir/POPG-CHARMM .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPG, "> popg-cg.pdb") || die "No output possible\n";

$popgid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq GLH){
			$popgid = $woe;	
			print POPG $spa,$woa,$spb,$wob,$spc,"C12",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popgid){
		if ($woc eq PO4){	
			print POPG $spa,$woa,$spb,$wob,$spc,"P1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPG $spa,$woa,$spb,$wob,$spc,"O21",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPG $spa,$woa,$spb,$wob,$spc,"O31",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C24",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C28",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C2A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C2B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C2C",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C34",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C3A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPG $spa,$woa,$spb,$wob,$spc,"C3B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPG);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f popg-cg.pdb -o popg-cg.pdb");

$num = 1;

while ($num <= $popg){

$loop = sprintf("%04d", $num);

open (ATM, "popg-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPG $popg";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $popg){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/GLH.pdb -name -o fit_${loop_main}/fit_GLH.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPG-CHARMM/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_GLH.pdb | grep 'P1' > headgroup_chemistry.pdb");
system("less fit_GLH.pdb | grep 'O1' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O2' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPG-CHARMM/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_GLH.pdb | grep ATOM | grep -v 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O3 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O4 ' >> complete_${loop_main}.pdb");
system("less fit_GLH.pdb | grep ATOM | grep 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'C3 ' | grep -v 'HX '| grep -v 'HY '| grep -v 'O31' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'O21'|grep -v 'C23' | grep -v 'H3R'| grep -v 'H3S'| grep -v 'C24'| grep -v 'H4R'| grep -v 'H4S' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'O2 ' | grep -v 'C1 '| grep -v 'HA '| grep -v 'HB ' | grep -v 'C2 '| grep -v 'HS '| grep -v 'O21'>> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C33' | grep -v 'O31'| grep -v 'H3X'| grep -v 'H3Y'| grep -v 'C34'| grep -v 'H4X'| grep -v 'H4Y' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'C21' | grep -v 'O21'| grep -v 'O22'| grep -v 'C22'| grep -v 'H2R'| grep -v 'H2S' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'C24' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' | grep -v 'C28' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'C2A' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'ATOM' | grep -v 'C2B' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C31' | grep -v 'O31'| grep -v 'O32'| grep -v 'C32'| grep -v 'H2X'| grep -v 'H2Y' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'C34' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' | grep -v 'C38' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'ATOM' | grep -v 'C3A' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > popg-bilayer.pdb");
system("cat popg-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPG-CHARMM/`;
}

#################### POPE ########################

if (($pope > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/POPE .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPE, "> pope-cg.pdb") || die "No output possible\n";

$popeid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NH3){
			$popeid = $woe;	
			print POPE $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popeid){
		if ($woc eq PO4){	
			print POPE $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPE $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPE $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"CA1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C49",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPE);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f pope-cg.pdb -o pope-cg.pdb");

$num = 1;

while ($num <= $pope){

$loop = sprintf("%04d", $num);

open (ATM, "pope-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPE $pope";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $pope){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/NH3.pdb -name -o fit_${loop_main}/fit_NH3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NH3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NH3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPE/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NH3.pdb | grep 'H1' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'H2' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'H3' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C50' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA1' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA2' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > pope-bilayer.pdb");
system("cat pope-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPE/`;
}

#################### POPE CHARMM ########################

if (($pope > 0)&&($ff =~ m/^CHARMM/)) {

`cp -R $cg2atdir/POPE-CHARMM .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPE, "> pope-cg.pdb") || die "No output possible\n";

$popeid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NH3){
			$popeid = $woe;	
			print POPE $spa,$woa,$spb,$wob,$spc,"N  ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popeid){
		if ($woc eq PO4){	
			print POPE $spa,$woa,$spb,$wob,$spc,"P1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPE $spa,$woa,$spb,$wob,$spc,"O21",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPE $spa,$woa,$spb,$wob,$spc,"O31",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C24",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C28",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C2A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C2B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C2C",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C34",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C3A",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPE $spa,$woa,$spb,$wob,$spc,"C3B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPE);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f pope-cg.pdb -o pope-cg.pdb");

$num = 1;

while ($num <= $pope){

$loop = sprintf("%04d", $num);

open (ATM, "pope-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPE $pope";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $pope){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/NH3.pdb -name -o fit_${loop_main}/fit_NH3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPE-CHARMM/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NH3.pdb | grep 'P1' > headgroup_chemistry.pdb");
system("less fit_NH3.pdb | grep 'O1' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O2' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPE-CHARMM/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NH3.pdb | grep ATOM | grep -v 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O3 ' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O4 ' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep ATOM | grep 'O1 ' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'C3 ' | grep -v 'HX '| grep -v 'HY '| grep -v 'O31' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'O21'|grep -v 'C23' | grep -v 'H3R'| grep -v 'H3S'| grep -v 'C24'| grep -v 'H4R'| grep -v 'H4S' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1' | grep 'ATOM'| grep -v 'O2 ' | grep -v 'C1 '| grep -v 'HA '| grep -v 'HB ' | grep -v 'C2 '| grep -v 'HS '| grep -v 'O21'>> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C33' | grep -v 'O31'| grep -v 'H3X'| grep -v 'H3Y'| grep -v 'C34'| grep -v 'H4X'| grep -v 'H4Y' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' | grep -v 'C21' | grep -v 'O21'| grep -v 'O22'| grep -v 'C22'| grep -v 'H2R'| grep -v 'H2S' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'C24' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' | grep -v 'C28' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'C2A' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'ATOM' | grep -v 'C2B' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C31' | grep -v 'O31'| grep -v 'O32'| grep -v 'C32'| grep -v 'H2X'| grep -v 'H2Y' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'C34' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' | grep -v 'C38' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'ATOM' | grep -v 'C3A' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > POPE-bilayer.pdb");
system("cat POPE-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPE-CHARMM/`;
}

#################### POPS ########################

if ($pops > 0) {

`cp -R $cg2atdir/POPS .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (POPS, "> pops-cg.pdb") || die "No output possible\n";

$popsid = 0;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq POP){
		if ($woc eq NCO){
			$popsid = $woe;	
			print POPS $spa,$woa,$spb,$wob,$spc,"C5 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $popsid){
		if ($woc eq PO4){	
			print POPS $spa,$woa,$spb,$wob,$spc,"P11",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print POPS $spa,$woa,$spb,$wob,$spc,"O17",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print POPS $spa,$woa,$spb,$wob,$spc,"O38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C22",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C32",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C36",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C43",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C47",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C51",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print POPS $spa,$woa,$spb,$wob,$spc,"C54",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}
}
close(EMI);
close(POPS);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f pops-cg.pdb -o pops-cg.pdb");

$num = 1;

while ($num <= $pops){

$loop = sprintf("%04d", $num);

open (ATM, "pops-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nPOPS $pops";
close (LIP);

open (POPS, ">> pops.top") || die "No output possible\n";
print POPS "\nPOPS $pops";
close (POPS);

# For every CG lipid
$num_main = 1;
while ($num_main <= $pops){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/NCO.pdb -name -o fit_${loop_main}/fit_NCO.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 POPS/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

`less fit_NCO.pdb | grep 'O10' > headgroup_chemistry.pdb`;
`less fit_NCO.pdb | grep 'P11' >> headgroup_chemistry.pdb`;
`less fit_CCP.pdb | grep 'O14' >> headgroup_chemistry.pdb`;

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../POPS/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

`less fit_NCO.pdb | grep -v 'P11' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'P11' >> complete_${loop_main}.pdb`;
`less fit_PO4.pdb | grep 'O12' >> complete_${loop_main}.pdb`;
`less fit_PO4.pdb | grep 'O13' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'O14' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C15' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C16' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'O17' >> complete_${loop_main}.pdb`;
`less fit_GL1.pdb | grep 'ATOM' | grep -v 'O17'| grep -v 'C22' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C22' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C23' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C24' >> complete_${loop_main}.pdb`;
`less fit_C1A.pdb | grep 'C25' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C26' >> complete_${loop_main}.pdb`;
`less fit_C2A.pdb | grep 'ATOM' | grep -v C26 | grep -v C29 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C29' >> complete_${loop_main}.pdb`;
`less fit_C3A.pdb | grep 'C30' >> complete_${loop_main}.pdb`;
`less fit_C3A.pdb | grep 'C31' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C32' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C33' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C34' >> complete_${loop_main}.pdb`;
`less fit_C4A.pdb | grep 'C35' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C36' >> complete_${loop_main}.pdb`;
`less fit_CCP.pdb | grep 'C37' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'O38' >> complete_${loop_main}.pdb`;
`less fit_GL2.pdb | grep 'ATOM' | grep -v O38 | grep -v C43 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C43' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C44' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C45' >> complete_${loop_main}.pdb`;
`less fit_C1B.pdb | grep 'C46' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C47' >> complete_${loop_main}.pdb`;
`less fit_C2B.pdb | grep 'ATOM' | grep -v C47 | grep -v C51 >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C51' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C52' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C53' >> complete_${loop_main}.pdb`;
`less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C54' >> complete_${loop_main}.pdb`;
`less fit_C3B.pdb | grep 'C55' >> complete_${loop_main}.pdb`;

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > pops-bilayer.pdb");
system("cat box.pdb pops-bilayer.pdb > pops-box.pdb");
system("$gromacs/grompp -f em.mdp -c pops-box.pdb -p pops.top -o pops_em -maxwarn 5"); 
system("$gromacs/mdrun -deffnm pops_em");
system("$gromacs/editconf -resnr 1 -f pops_em.gro -o pops_em.pdb");
system("cat pops_em.pdb | grep ATOM >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ POPS/`;
}

#################### DOPC GROMOS ########################

if (($dopc > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/DOPC .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DOPC, "> dopc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DOP){
		if ($woc eq NC3){
			print DOPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C45",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C48",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"CA1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		if ($woc eq C5A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"CA3",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}

close(EMI);
close(DOPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dopc-cg.pdb -o dopc-cg.pdb");

$num = 1;

while ($num <= $dopc){

$loop = sprintf("%04d", $num);

open (ATM, "dopc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDOPC $dopc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dopc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC/C4B.pdb -name -o fit_${loop_main}/fit_C4B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DOPC/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C46' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C50' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'CA1' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'CA2' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA3' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA4' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dopc-bilayer.pdb");
system("cat dopc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DOPC/`;
}

#################### DOPC GROMOS END ####################

#################### DOPC OPLSUA ########################

if (($dopc > 0)&&($ff eq "OPLSUA")) {

`cp -R $cg2atdir/DOPC-oplsua .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DOPC, "> dopc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DOP){
		if ($woc eq NC3){
			print DOPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"O35",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C40",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C44",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C47",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C50",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5B){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C53",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq D3A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		if ($woc eq C5A){	
			print DOPC $spa,$woa,$spb,$wob,$spc,"C32",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}

close(EMI);
close(DOPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dopc-cg.pdb -o dopc-cg.pdb");

$num = 1;

while ($num <= $dopc){

$loop = sprintf("%04d", $num);

open (ATM, "dopc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDOPC $dopc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dopc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DOPC-oplsua/C4B.pdb -name -o fit_${loop_main}/fit_C4B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DOPC-oplsua/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C34' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C42' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C43' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dopc-bilayer.pdb");
system("cat dopc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DOPC-oplsua/`;
}

#################### DOPC OPLSUA END ####################

#################### DSPE GROMOS ########################

if (($dspe > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/DSPE .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DSPE, "> dspe-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DSP){
		if ($woc eq NH3){
			print DSPE $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C45",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C48",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C5A){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"CA3",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C26",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		if ($woc eq C5B){	
			print DSPE $spa,$woa,$spb,$wob,$spc,"CA1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}

close(EMI);
close(DSPE);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dspe-cg.pdb -o dspe-cg.pdb");

$num = 1;

while ($num <= $dspe){

$loop = sprintf("%04d", $num);

open (ATM, "dspe-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDSPE $dspe";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dspe){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/NH3.pdb -name -o fit_${loop_main}/fit_NH3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C4A.pdb -name -o fit_${loop_main}/fit_C4A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DSPE/C4B.pdb -name -o fit_${loop_main}/fit_C4B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NH3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NH3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DSPE/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NH3.pdb | grep 'H1' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'H2' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'H3' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NH3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C27' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C46' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'C50' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA1' >> complete_${loop_main}.pdb");
system("less fit_C4A.pdb | grep 'CA2' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'CA3' >> complete_${loop_main}.pdb");
system("less fit_C4B.pdb | grep 'CA4' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dspe-bilayer.pdb");
system("cat dspe-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DSPE/`;
}

#################### DSPE GROMOS END ####################


#################### DPPC GROMOS ########################

if (($dppc > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/DPPC .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DPPC, "> dppc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DPP){
		if ($woc eq NC3){
			print DPPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"O33",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C38",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C42",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C49",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C27",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C30",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}

close(EMI);
close(DPPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dppc-cg.pdb -o dppc-cg.pdb");

$num = 1;

while ($num <= $dppc){

$loop = sprintf("%04d", $num);

open (ATM, "dppc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDPPC $dppc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dppc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");


system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");


chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DPPC/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'C31' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C32' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C41' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C47' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C48' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C49' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep 'C50' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dppc-bilayer.pdb");
system("cat dppc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DPPC/`;
}

######################DPPC_END#########################

#################### DPPC OPLSAA ########################

if (($dppc > 0)&&($ff =~ m/^OPLSAA/)) {

`cp -R $cg2atdir/DPPC-oplsaa .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DPPC, "> dppc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DPP){
		if ($woc eq NC3){
			print DPPC $spa,$woa,$spb,$wob,$spc,"N1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"P1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"O5 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"O7 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C28",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C32",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C36",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4B){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C40",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C11",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C15",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C4A){	
			print DPPC $spa,$woa,$spb,$wob,$spc,"C23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}			
		}
	}

close(EMI);
close(DPPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dppc-cg.pdb -o dppc-cg.pdb");

$num = 1;

while ($num <= $dppc){

$loop = sprintf("%04d", $num);

open (ATM, "dppc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDPPC $dppc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dppc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C3A.pdb -name -o fit_${loop_main}/fit_C3A.pdb << EOD
1
1
EOD");


system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DPPC-oplsaa/C3B.pdb -name -o fit_${loop_main}/fit_C3B.pdb << EOD
1
1
EOD");


chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O1' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P1' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O4' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DPPC-oplsaa/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_C3A.pdb | grep 'C23' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep -v 'C23' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep -v 'C19' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep -v 'C15' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep -v 'C11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C7' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep -v 'P1' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O2' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O3' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'P1'| grep -v 'O5'| grep -v 'C7' | grep -v 'C6'| grep -v 'O4'>> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep -v 'O7' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep -v 'C28' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep -v 'C32' >> complete_${loop_main}.pdb");
system("less fit_C3B.pdb | grep -v 'C36' >> complete_${loop_main}.pdb");


chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dppc-bilayer.pdb");
system("cat dppc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DPPC-oplsaa/`;
}

######################DPPC_END#########################

#################### DMPC ########################

if ($dmpc > 0) {

`cp -R $cg2atdir/DMPC .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DMPC, "> dmpc-cg.pdb") || die "No output possible\n";

$dmpcid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DLP){
		if ($woc eq NC3){
			$dmpcid = $woe;	
			print DMPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dmpcid){
		if ($woc eq PO4){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"O31",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C36",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C41",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C24",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print DMPC $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
}
close(EMI);
close(DMPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dmpc-cg.pdb -o dmpc-cg.pdb");

$num = 1;

while ($num <= $dmpc){

$loop = sprintf("%04d", $num);

open (ATM, "dmpc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDMPC $dmpc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dmpc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPC/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DMPC/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C23' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C37' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C38' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dmpc-bilayer.pdb");
system("cat dmpc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DMPC/`;
}

######################DMPC_END####################

#################### DMPG ########################

if ($dmpg > 0) {

`cp -R $cg2atdir/DMPG .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DMPG, "> dmpg-cg.pdb") || die "No output possible\n";

$dmpgid = 0 ;

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DLP){
		if ($woc eq GLH){
			$dmpgid = $woe;	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C2 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ($woe eq $dmpgid){			
		if ($woc eq PO4){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"O31",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C36",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C41",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3B){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C46",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C19",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C24",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3A){	
			print DMPG $spa,$woa,$spb,$wob,$spc,"C29",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
}
close(EMI);
close(DMPG);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dmpg-cg.pdb -o dmpg-cg.pdb");

$num = 1;

while ($num <= $dmpg){

$loop = sprintf("%04d", $num);

open (ATM, "dmpg-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDMPG $dmpg";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dmpg){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/GLH.pdb -name -o fit_${loop_main}/fit_GLH.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DMPG/C2B.pdb -name -o fit_${loop_main}/fit_C2B.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_GLH.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_GLH.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DMPG/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

`less fit_GLH.pdb | grep 'O1' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'C2' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'C3' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'O4' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'H5' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'C6' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'O7' >> complete_${loop_main}.pdb`;
`less fit_GLH.pdb | grep 'P8' >> complete_${loop_main}.pdb`;
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C23' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C30' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C37' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C38' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C39' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C40' >> complete_${loop_main}.pdb");
system("less fit_C2B.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
`less fit_GLH.pdb | grep 'H0' >> complete_${loop_main}.pdb`;

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dmpg-bilayer.pdb");
system("cat dmpg-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DMPG/`;
}

######################DMPG_END#########################

#################### DHPC GROMOS ########################

if (($dhpc > 0)&&($ff ne "OPLSAA")) {

`cp -R $cg2atdir/DHPC .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DHPC, "> dhpc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DHP){
		if ($woc eq NC3){
			print DHPC $spa,$woa,$spb,$wob,$spc,"N4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"P8 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"O14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"O23",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C27",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C30",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C18",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C21",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(DHPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dhpc-cg.pdb -o dhpc-cg.pdb");

$num = 1;

while ($num <= $dhpc){

$loop = sprintf("%04d", $num);

open (ATM, "dhpc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDHPC $dhpc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dhpc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");


chdir "fit_${loop_main}";

system("less fit_NC3.pdb | grep 'O7' > headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'P8' >> headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'O11' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DHPC/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_NC3.pdb | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C2' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C3' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'N4' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C5' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'C6' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep 'P8' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O9' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O10' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'O11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C12' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C13' >> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C19' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C20' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'C21' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep 'C22' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C28' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C29' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'C30' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dhpc-bilayer.pdb");
system("cat dhpc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DHPC/`;
}

##################DHPC GROMOS END#########################

################### DHPC OPLS-AA #########################

if (($dhpc > 0)&&($ff eq "OPLSAA")) {

`cp -R $cg2atdir/DHPC-oplsaa .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DHPC, "> dhpc-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DHP){
		if ($woc eq NC3){
			print DHPC $spa,$woa,$spb,$wob,$spc,"N1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq PO4){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"P1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL1){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"O3 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq GL2){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"O2 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1B){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C4 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2B){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1A){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C11",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2A){	
			print DHPC $spa,$woa,$spb,$wob,$spc,"C14",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(DHPC);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f dhpc-cg.pdb -o dhpc-cg.pdb");

$num = 1;

while ($num <= $dhpc){

$loop = sprintf("%04d", $num);

open (ATM, "dhpc-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDHPC $dhpc";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $dhpc){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/NC3.pdb -name -o fit_${loop_main}/fit_NC3.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/CCP.pdb -name -o fit_${loop_main}/fit_CCP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/GL1.pdb -name -o fit_${loop_main}/fit_GL1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/GL2.pdb -name -o fit_${loop_main}/fit_GL2.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DHPC-oplsaa/C1B.pdb -name -o fit_${loop_main}/fit_C1B.pdb << EOD
1
1
EOD");


chdir "fit_${loop_main}";

system("less fit_CCP.pdb | grep 'O5' > headgroup_chemistry.pdb");
system("less fit_CCP.pdb | grep 'P1' >> headgroup_chemistry.pdb");
system("less fit_NC3.pdb | grep 'O8' >> headgroup_chemistry.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 headgroup_chemistry.pdb -f2 ../DHPC-oplsaa/PO4.pdb -name -o fit_PO4.pdb << EOD
1
1
EOD");

system("less fit_C1B.pdb | grep 'ATOM' | grep -v 'C4' >> complete_${loop_main}.pdb");
system("less fit_C1B.pdb | grep 'ATOM' | grep 'C4' >> complete_${loop_main}.pdb");
system("less fit_GL2.pdb | grep 'ATOM' | grep -v 'C4' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'C15' | grep -v 'H26'| grep -v 'H27'| grep -v 'O5'| grep -v 'P1'|grep -v 'O2'>> complete_${loop_main}.pdb");
system("less fit_GL1.pdb | grep -v 'O3' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep -v 'C11' >> complete_${loop_main}.pdb");
system("less fit_CCP.pdb | grep -v 'C7' | grep -v 'H12'| grep -v 'H13'| grep -v 'C8'| grep -v 'H14'|grep -v 'O3'|grep -v 'O2'>> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O6' >> complete_${loop_main}.pdb");
system("less fit_PO4.pdb | grep 'O7' >> complete_${loop_main}.pdb");
system("less fit_NC3.pdb | grep -v 'P1' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > dhpc-bilayer.pdb");
system("cat dhpc-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DHPC-oplsaa/`;
}

############### DHPC OPLS END#########################

#################### BOG ########################

if (($bog > 0)&&($ff =~ m/^GROMOS/)) {

`cp -R $cg2atdir/BOG .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (BOG, "> bog-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq BOG){
		if ($woc eq B1){
			print BOG $spa,$woa,$spb,$wob,$spc,"C5 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B2){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C3 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B3){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C1 ",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C4B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C8B",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(BOG);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f bog-cg.pdb -o bog-cg.pdb");

$num = 1;

while ($num <= $bog){

$loop = sprintf("%04d", $num);

open (ATM, "bog-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nBOG $bog";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $bog){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG/RP.pdb -name -o fit_${loop_main}/fit_RP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_C2A.pdb | grep 'ATOM' | grep -v 'C4B' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'O1'| grep -v 'C1 ' >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'ATOM' | grep 'O1' >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'ATOM' | grep 'C1' >> complete_${loop_main}.pdb");
system("less fit_RP.pdb | grep 'ATOM' | grep -v 'O1' | grep -v 'C1' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > bog-bilayer.pdb");
system("cat bog-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ BOG/`;
}

######################BOG_END#########################

#################### BOG ########################

if (($bog > 0)&&($ff eq "OPLSAA")) {

`cp -R $cg2atdir/BOG-oplsaa .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (BOG, "> bog-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq BOG){
		if ($woc eq B1){
			print BOG $spa,$woa,$spb,$wob,$spc,"C5",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B2){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C3",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B3){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C1",$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C10 ",$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2){	
			print BOG $spa,$woa,$spb,$wob,$spc,"C14 ",$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(BOG);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f bog-cg.pdb -o bog-cg.pdb");

$num = 1;

while ($num <= $bog){

$loop = sprintf("%04d", $num);

open (ATM, "bog-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nBOG $bog";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $bog){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG-oplsaa/RP.pdb -name -o fit_${loop_main}/fit_RP.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG-oplsaa/C1A.pdb -name -o fit_${loop_main}/fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 BOG-oplsaa/C2A.pdb -name -o fit_${loop_main}/fit_C2A.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_RP.pdb | grep 'ATOM'  >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'C1' |  grep -v 'C10' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > bog-bilayer.pdb");
system("cat bog-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
#`rm '#'*`;
#`rm -rf Res_CG/ fit_*/ BOG-oplsaa/`;
}

######################BOG_END#########################

################### CHOLESTEROL ########################

if ($chol > 0) {

`cp -R $cg2atdir/CHOL .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (CHO, "> chol-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq CHO){
		if ($woc eq ROH){
			print CHO $spa,$woa,$spb,$wob,$spc,"C5  CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq R1){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C5  CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq R2){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C11 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq R3){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C14 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq R4){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C19 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq R5){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C16 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C24 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2){	
			print CHO $spa,$woa,$spb,$wob,$spc,"C27 CHO",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(CHO);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f chol-cg.pdb -o chol-cg.pdb");

$num = 1;

while ($num <= $chol){

$loop = sprintf("%04d", $num);

open (ATM, "chol-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nCHOL $chol";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $chol){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 CHOL/RP.pdb -name -o fit_${loop_main}/fit_RP.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";

system("less fit_RP.pdb | grep 'ATOM' > chol-rings+tail.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C24' >> chol-rings+tail.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C27' >> chol-rings+tail.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 chol-rings+tail.pdb -f2 ../CHOL/C1A.pdb -name -o fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 chol-rings+tail.pdb -f2 ../CHOL/C2A.pdb -name -o fit_C2A.pdb << EOD
1
1
EOD");

system("less fit_RP.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'C16'| grep -v 'C20'| grep -v 'C21'| grep -v 'C24' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > chol-bilayer.pdb");
system("cat chol-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ CHOL/`;
}

######################CHOL_END#########################

################### DDM ########################

if ($ddm > 0) {

`cp -R $cg2atdir/DDM .`;

open (EMI, "em_inbox.pdb") || die "Cannot open em_inbox.pdb\n";
open (DDM, "> ddm-cg.pdb") || die "No output possible\n";

while (<EMI>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($wod eq DDM){
		if ($woc eq B1){
			print DDM $spa,$woa,$spb,$wob,$spc,"C5  DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B2){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C1  DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B3){	
			print DDM $spa,$woa,$spb,$wob,$spc,"O2  DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B4){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C7  DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B5){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C11 DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq B6){	
			print DDM $spa,$woa,$spb,$wob,$spc,"O7  DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C1){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C16 DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C2){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C20 DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		if ($woc eq C3){	
			print DDM $spa,$woa,$spb,$wob,$spc,"C23 DDM",$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		}
	}
	
close(EMI);
close(DDM);

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');
system("$gromacs/editconf -resnr 1 -f ddm-cg.pdb -o ddm-cg.pdb");

$num = 1;

while ($num <= $ddm){

$loop = sprintf("%04d", $num);

open (ATM, "ddm-cg.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $num){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);
close(RESCG);

$num++;
}

open (LIP, ">> lipid.top") || die "No output possible\n";
print LIP "\nDDM $ddm";
close (LIP);

# For every CG lipid
$num_main = 1;
while ($num_main <= $ddm){

$loop_main = sprintf("%04d", $num_main);

# Temporary directory to store all fitted AT lipids in library to the current CG lipid
mkdir("fit_${loop_main}");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DDM/RP1.pdb -name -o fit_${loop_main}/fit_RP1.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 DDM/RP2.pdb -name -o fit_${loop_main}/fit_RP2.pdb << EOD
1
1
EOD");

chdir "fit_${loop_main}";
system("cat fit_RP1.pdb fit_RP2.pdb | grep 'ATOM' > ddm-rings+tail.pdb");

system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C16' >> ddm-rings+tail.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C20' >> ddm-rings+tail.pdb");
system("less ../Res_CG/res_cg_${loop_main}.pdb | grep 'C23' >> ddm-rings+tail.pdb");

system("$gromacs/g_confrms -nice 0 -one -f1 ddm-rings+tail.pdb -f2 ../DDM/C1A.pdb -name -o fit_C1A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 ddm-rings+tail.pdb -f2 ../DDM/C2A.pdb -name -o fit_C2A.pdb << EOD
1
1
EOD");

system("$gromacs/g_confrms -nice 0 -one -f1 ddm-rings+tail.pdb -f2 ../DDM/C3A.pdb -name -o fit_C3A.pdb << EOD
1
1
EOD");

system("less fit_RP1.pdb | grep 'ATOM'| grep -v C1 |grep -v H9|grep -v H2| grep -v 'H1 '|grep -v O1|grep -v O2 >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep O2 >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep H9 >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep H2 >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep 'C1 ' >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep 'H1 ' >> complete_${loop_main}.pdb");
system("less fit_RP1.pdb | grep 'ATOM'| grep O1 >> complete_${loop_main}.pdb");
system("less fit_RP2.pdb | grep 'ATOM'| grep -v C7 |grep -v H12|grep -v O4 >> complete_${loop_main}.pdb");
system("less fit_RP2.pdb | grep 'ATOM'| grep C7 >> complete_${loop_main}.pdb");
system("less fit_RP2.pdb | grep 'ATOM'| grep H12 >> complete_${loop_main}.pdb");
system("less fit_RP2.pdb | grep 'ATOM'| grep O4 >> complete_${loop_main}.pdb");
system("less fit_C1A.pdb | grep 'ATOM' | grep -v 'O4' >> complete_${loop_main}.pdb");
system("less fit_C2A.pdb | grep 'ATOM' | grep -v 'C16' >> complete_${loop_main}.pdb");
system("less fit_C3A.pdb | grep 'ATOM' | grep -v 'C20' >> complete_${loop_main}.pdb");

chdir "..";

$num_main++;
}

system("cat fit_*/complete_*.pdb | grep 'ATOM' > ddm-bilayer.pdb");
system("cat ddm-bilayer.pdb >> bilayer-box.pdb");

# Cleaning up...
`rm '#'*`;
`rm -rf Res_CG/ fit_*/ DDM/`;
}

######################DDM_END#########################

if ($lipidnum > 0) {
system("$gromacs/grompp_d -maxwarn 10 -f em.mdp -o lipid_em -c bilayer-box.pdb -p lipid.top");
system("$gromacs/mdrun_d -deffnm lipid_em");
system("$gromacs/editconf -resnr 1 -f lipid_em.gro -o atomistic-lipid.pdb");
open ATM, "<", 'atomistic-lipid.pdb' or die "Energy minimisation of AT lipid failed\n";
close(ATM);
}

#######################PROTEIN#########################

if ($protnum > 0) {

`cp -R $cg2atdir/Sidechains .`;

system("$gromacs/make_ndx -f em_inbox.pdb -o full.ndx << EOD
del 0
del 1-15
q
EOD");

system("$gromacs/editconf -resnr 1 -f em_inbox.pdb -o protein.pdb -n full.ndx");

open PRO,"<", 'protein.pdb' or die;
open ATM, ">", 'atomistic-particles.pdb' or die;

if ($forcefield eq "martini") { 

$count=1;

while (<PRO>){
		chomp;
		($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
		($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
		if ($woa eq "ATOM"){
		 if (($woc=~ m/^B/)||($woc=~ m/^0B/)||($woc=~ m/^4B/)||($woc=~ m/^5B/)){$woc=" CA ";
		 	} elsif ($wod eq "CYS"){$woc=" SG ";
		 	} elsif (($wod eq "GLU")||($wod eq "GLN")) {$woc=" CD ";
			} elsif ($wod eq "ILE"){$woc=" CG1";
			} elsif ($wod eq "MET"){$woc=" SD ";
			} elsif ($wod eq "VAL"){$woc=" CB ";
			} elsif ($wod eq "SER"){$woc=" OG ";
			} elsif ($wod eq "THR"){$woc=" OG1";
			} elsif (($wod eq "ARG")&&($woc=~ m/Qd/)){$woc=" NE ";
			} elsif (($wod eq "LYS")&&($woc=~ m/Qd/)){$woc=" CE ";
			} elsif (($wod eq "HIS")&&($woc=~ m/^S/)&&($count==1)){$woc=" CG ";
				$count++;
			} elsif (($wod eq "HIS")&&($woc=~ m/^S/)&&($count==2)){$woc=" ND1";
				$count++;
			} elsif (($wod eq "HIS")&&($woc=~ m/^S/)&&($count==3)){$woc=" NE2";
				$count=1;
			} elsif (($wod eq "TYR")&&($woc=~ m/^S/)&&($count==1)){$woc=" CG ";
				$count++;
			} elsif (($wod eq "TYR")&&($woc=~ m/^S/)&&($count==2)){$woc=" CE1";
				$count++;
			} elsif (($wod eq "TYR")&&($woc=~ m/^S/)&&($count==3)){$woc=" CE2";
				$count=1;
			} elsif (($wod eq "PHE")&&($woc=~ m/^S/)&&($count==1)){$woc=" CG ";
				$count++;
			} elsif (($wod eq "PHE")&&($woc=~ m/^S/)&&($count==2)){$woc=" CE1";
				$count++;
			} elsif (($wod eq "PHE")&&($woc=~ m/^S/)&&($count==3)){$woc=" CE2";
				$count=1;
			} elsif (($wod eq "TRP")&&($woc=~ m/^S/)&&($count==1)){$woc=" CG ";
				$count++;
			} elsif (($wod eq "TRP")&&($woc=~ m/^S/)&&($count==2)){$woc=" CE2";
				$count++;
			} elsif (($wod eq "TRP")&&($woc=~ m/^S/)&&($count==3)){$woc=" CE3";
				$count++;
			} elsif (($wod eq "TRP")&&($woc=~ m/^S/)&&($count==4)){$woc=" CH2";
				$count=1;
				} else {$woc=" CG ";
			}
		print ATM $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
		} else {
		print ATM $_,"\n";
		}
	}
close (PRO);
close (ATM);
}
elsif ($forcefield eq "bond") { 

my @prot = <PRO>;
s/CB  ASP/CG  ASP/g for @prot;
s/CB  GLU/CD  GLU/g for @prot;
s/CB  PHE/CE1 PHE/g for @prot;
s/CG  PHE/CE2 PHE/g for @prot;
s/CB  HIS/CD2 HIS/g for @prot;
s/CG  HIS/ND1 HIS/g for @prot;
s/CB  ILE/CG1 ILE/g for @prot;
s/CB  LEU/CG  LEU/g for @prot;
s/CG  ARG/CZ  ARG/g for @prot;
s/CB  ARG/CG  ARG/g for @prot;
s/CB  SER/OG  SER/g for @prot;
s/CB  LYS/CG  LYS/g for @prot;
s/CG  LYS/NZ  LYS/g for @prot;
s/CB  TRP/CE3 TRP/g for @prot;
s/CG  TRP/NE1 TRP/g for @prot;
s/CB  CYS/SG  CYS/g for @prot;
s/CB  MET/SD  MET/g for @prot;
s/CB  GLN/CD  GLN/g for @prot;
s/CB  ASN/CG  ASN/g for @prot;
s/CB  THR/OG1 THR/g for @prot;
s/CB  VAL/CB  VAL/g for @prot;
s/CB  TYR/CE1 TYR/g for @prot;
s/CG  TYR/CE2 TYR/g for @prot;
s/CB  PRO/CG  PRO/g for @prot;
print ATM @prot;
close (ATM);
close (PRO);
}

open ATM, "<", 'atomistic-particles.pdb' or die;

$resnum = 0;

while (<ATM>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	if (($woc eq CA)&&($woa eq ATOM)){
	$resnum++;
		}
	}
close (ATM);	

if ($method eq "align"){

system("$gromacs/make_ndx -f cg2atalignfile.pdb -o nohydrogens.ndx << EOD
del 0-1
del 1-20
q
EOD
");

system("$gromacs/editconf -resnr 1 -f cg2atalignfile.pdb -o nohydrogens.pdb -n nohydrogens.ndx << EOD
0
EOD
");

system("$gromacs/editconf -resnr 1 -f nohydrogens.pdb -o labelZ.pdb -label Z");

open LABZ, "<", 'labelZ.pdb' or die;
open NOLAB, ">", 'nolabel.pdb' or die;

my @z = <LABZ>;
s/ Z/  /g for @z;
print NOLAB @z;

close (LABZ);
close (NOLAB);

if ($protnum > 1){

system("rm chain_*.pdb");

@chainid = 1 .. $protnum;

@chain_end = map($_-1,@chain_num);
push (@chain_end,$resnum);

system("$gromacs/make_ndx -f atomistic-particles.pdb -o split-chain.ndx << EOD
del 1-30
r${chain_num[0]}-${chain_end[1]}&aCA
q
EOD
");

for($i = 1; $i <= $protnum ; $i++) {
system("$gromacs/make_ndx -f atomistic-particles.pdb -n split-chain.ndx  -o split-chain.ndx << EOD
r${chain_num[$i]}-${chain_end[$i+1]}&aCA
q
EOD");
}

for($i = 0; $i < $protnum ; $i++) {
system("$gromacs/g_confrms -nice 0 -one -f1 atomistic-particles.pdb -f2 nolabel.pdb -name -o chain_${chainid[$i]}.pdb -n1 split-chain.ndx << EOD
$i
3
EOD");
}

system("cat chain_*.pdb | grep ATOM > protein_aligned.pdb");

system("$gromacs/editconf -resnr 1 -f protein_aligned.pdb -o protein-sorted.pdb");

open SID, "<", 'protein-sorted.pdb' or die;
open CHA, ">", 'aligned_chain.pdb' or die;

$oldres=-999;
$nchain=0;
$chain=();
@refchain = 'A' .. 'Z';

while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,27,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain] ;         
             $nchain++;
             }
           if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } else {print CHA $_,"\n"};
	   $oldres=$res;
           	}
close(CHA);
close(SID);

open OPT, ">", 'optimize.py' or die;
print OPT "\nfrom modeller import *";
print OPT "\nfrom modeller.scripts import complete_pdb";
print OPT "\nfrom modeller.optimizers import conjugate_gradients, molecular_dynamics, actions";
print OPT "\nenv = environ()";
print OPT "\nenv.io.atom_files_directory = ['../atom_files']";
print OPT "\nenv.edat.dynamic_sphere = True";
print OPT "\nenv.libs.topology.read(file='\$(LIB)/top_heav.lib')";
print OPT "\nenv.libs.parameters.read(file='\$(LIB)/par.lib')";
print OPT "\ncode = 'aligned_chain.pdb'";
print OPT "\nmdl = complete_pdb(env, code)";
print OPT "\nmdl.write(file=code+'.ini')";
print OPT "\natmsel = selection(mdl)";
print OPT "\nmdl.restraints.make(atmsel, restraint_type='stereo', spline_on_site=False)";
print OPT "\nmdl.restraints.write(file=code+'.rsr')";
print OPT "\nmpdf = atmsel.energy()";
print OPT "\ncg = conjugate_gradients(output='REPORT')";
print OPT "\nmd = molecular_dynamics(output='REPORT')";
print OPT "\ntrcfil = file(code+'.D00000001', 'w')";
print OPT "\ncg.optimize(atmsel, max_iterations=5)";
print OPT "\nmd.optimize(atmsel, temperature=300, max_iterations=50,";
print OPT "\nactions=[actions.write_structure(10, code+'.D9999%04d.pdb'),";
print OPT "\nactions.trace(10, trcfil)])";
print OPT "\ncg.optimize(atmsel, max_iterations=20,";
print OPT "\nactions=[actions.trace(5, trcfil)])";


print OPT "\nmpdf = atmsel.energy()";
print OPT "\nmdl.write(file='model.pdb')";
close(OPT);

system("$modeller optimize.py");
}

if ($protnum == 1){

open SID, "<", 'nolabel.pdb' or die;
open CHA, ">", 'chainlabel.pdb' or die;

$oldres=-999;
$nchain=0;
$chain=();
@refchain='A' .. 'Z'; 

while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,26,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain];         
             $nchain++;
             }
           if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } ;
	   $oldres=$res;
           }
close(CHA);
close(SID);

system("$gromacs/g_confrms -nice 0 -one -f1 atomistic-particles.pdb -f2 chainlabel.pdb -name -o model.pdb << EOD
3
3
EOD");
	}
}
elsif ($method eq "full"){

`rm -rf Res_CG fit_*`;
mkdir('Res_CG');

$loop = 1;

while ($loop <= $resnum){

open (ATM, "atomistic-particles.pdb") || die "Cannot open atomistic-particles.pdb\n";
open (RESCG, "> Res_CG/res_cg_${loop}.pdb") || die "No output possible in Res_CG\n";

while (<ATM>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe eq $loop){
		if ((length($woe))==4){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"   1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
			}
		elsif ((length($woe))==3){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"  1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==2){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe," 1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		elsif ((length($woe))==1){
			print RESCG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,"1",$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n";
			}
		}
	}			
close(ATM);

print RESCG "ATOM      3  DA  PRO     1       1.000   1.000   1.000  1.00  0.00","\n" ;

close(RESCG);

$loop++;
}

$loop_main = 1;

while ($loop_main <= $resnum){

mkdir("fit_${loop_main}");

open (RES, "< Res_CG/res_cg_${loop_main}.pdb") || die;
open (CAF, "> fit_${loop_main}/residue_${loop_main}.pdb") || die "No output possible\n";

while (<RES>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if (($woc eq CA)&&(($wod eq "ALA")||($wod eq "CYS")||($wod eq "GLU")||($wod eq "GLN")||($wod eq "ASN")||($wod eq "ARG")||($wod eq "VAL")||($wod eq "ASP")||($wod eq "ILE")||($wod eq "LEU")||($wod eq "MET")||($wod eq "GLY")||($wod eq "LYS")||($wod eq "SER")||($wod eq "THR"))){
	`cp Res_CG/res_cg_${loop_main}.pdb fit_${loop_main}/fit_${loop_main}.pdb`;
	}	
	elsif (($woc eq CA)&&($wod eq "PRO")) {
	system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 Sidechains/PRO.pdb -name -o fit_${loop_main}/fit_${loop_main}.pdb << EOD
	1
	1
	EOD");
	}
	elsif (($woc eq CA)&&($wod eq "PHE")) {
	system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 Sidechains/PHE.pdb -name -o fit_${loop_main}/fit_${loop_main}.pdb << EOD
	1
	1
	EOD");
	}
	elsif (($woc eq CA)&&($wod eq "HIS")) {
	system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 Sidechains/HIS.pdb -name -o fit_${loop_main}/fit_${loop_main}.pdb << EOD
	1
	1
	EOD");
	}
	elsif (($woc eq CA)&&($wod eq "TRP")) {
	system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 Sidechains/TRP.pdb -name -o fit_${loop_main}/fit_${loop_main}.pdb << EOD
	1	
	1
	EOD");
	}
	elsif (($woc eq CA)&&($wod eq "TYR")) {
	system("$gromacs/g_confrms -nice 0 -one -f1 Res_CG/res_cg_${loop_main}.pdb -f2 Sidechains/TYR.pdb -name -o fit_${loop_main}/fit_${loop_main}.pdb << EOD
	1
	1
	EOD");
	}
	if ($woc eq "CA") {
	print CAF $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
}

open (FIT, "< fit_${loop_main}/fit_${loop_main}.pdb") || die;

while (<FIT>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if (($woa eq ATOM)&&(($woc ne CA)&&($woc ne DA))) {
	print CAF $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
}
close (FIT);
close (CAF);
close (RES);

open (RSL, "< fit_${loop_main}/residue_${loop_main}.pdb") || die;
open RSN, ">>", 'protein_sidechains.pdb' or die;

while (<RSL>) {
           $len=length($_);
           $str1=(substr($_,0,22));
	   $str2=(substr($_,26,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	   if ($woa eq "ATOM") {
            if (length($loop_main)==4) {
             print RSN $str1,"",$loop_main,"",$str2;
             } elsif (length($loop_main)==3) {
             	print RSN $str1," ",$loop_main,"",$str2;
             	} elsif (length($loop_main)==2) {
                    print RSN $str1,"  ",$loop_main,"",$str2;
            		} elsif (length($loop_main)==1) {
             		print RSN $str1,"   ",$loop_main,"",$str2;
           		 } else {print RSN $_,"\n"};
           	}
	}
close(RSL);
close(RSN);

$loop_main++;
}

open SID, "<", 'protein_sidechains.pdb' or die;
open CHA, ">", 'protein_chain.pdb' or die;

$oldres=-999;
$nchain=0;
$chain=();
@refchain='A' .. 'Z'; 
  
while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,27,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain];         
             $nchain++;
             }
           if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } else {print CHA $_,"\n"};
	   $oldres=$res;
           }
close(CHA);
close(SID);

open (PCH, "protein_chain.pdb") || die "Cannot open protein_chain.pdb\n";
open CHA, ">", 'chainA.pdb' or die;
open CHB, ">", 'chainB.pdb' or die;
open CHC, ">", 'chainC.pdb' or die;
open CHD, ">", 'chainD.pdb' or die;
open CHE, ">", 'chainE.pdb' or die;
open CHF, ">", 'chainF.pdb' or die;
open CHG, ">", 'chainG.pdb' or die;
open CHH, ">", 'chainH.pdb' or die;
open CHI, ">", 'chainI.pdb' or die;
open CHJ, ">", 'chainJ.pdb' or die;
open CHK, ">", 'chainK.pdb' or die;
open CHL, ">", 'chainL.pdb' or die;
open CHM, ">", 'chainM.pdb' or die;
open CHN, ">", 'chainN.pdb' or die;
open CHO, ">", 'chainO.pdb' or die;
open CHP, ">", 'chainP.pdb' or die;

while (<PCH>){
	chomp;
	($spa, $spb, $spc, $spd, $spe, $spf, $spg, $sph, $spi,$spj) = split(/\S+/, $_);
	($woa, $wob, $woc, $wod, $woe, $wof, $wog, $woh, $woi, $woj) = split;
	if ($woe=~ m/^A/){
	print CHA $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^B/){
	print CHB $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^C/){
	print CHC $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^D/){
	print CHD $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^E/){
	print CHE $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^F/){
	print CHF $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^G/){
	print CHG $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^H/){
	print CHH $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^I/){
	print CHI $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^J/){
	print CHJ $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^K/){
	print CHK $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^L/){
	print CHL $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^M/){
	print CHM $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^N/){
	print CHN $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^O/){
	print CHO $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
	if ($woe=~ m/^P/){
	print CHP $spa,$woa,$spb,$wob,$spc,$woc,$spd,$wod,$spe,$woe,$spf,$wof,$spg,$wog,$sph,$woh,$spi,$woi,$spj,$woj,"\n" ;
	}
}
close (PCH);
close (CHA);
close (CHB);
close (CHC);
close (CHD);
close (CHE);
close (CHF);
close (CHG);
close (CHH);
close (CHI);
close (CHJ);
close (CHK);
close (CHL);
close (CHM);
close (CHN);
close (CHO);
close (CHP);

system("$pulchra -s chainA.pdb");
system("$pulchra -s chainB.pdb");
system("$pulchra -s chainC.pdb");
system("$pulchra -s chainD.pdb");
system("$pulchra -s chainE.pdb");
system("$pulchra -s chainF.pdb");
system("$pulchra -s chainG.pdb");
system("$pulchra -s chainH.pdb");
system("$pulchra -s chainI.pdb");
system("$pulchra -s chainJ.pdb");
system("$pulchra -s chainK.pdb");
system("$pulchra -s chainL.pdb");
system("$pulchra -s chainM.pdb");
system("$pulchra -s chainN.pdb");
system("$pulchra -s chainO.pdb");
system("$pulchra -s chainP.pdb");

system("cat chain*.rebuilt.pdb | grep ATOM > protein_pulchra.pdb");

open SID, "<", 'protein_pulchra.pdb' or die;
open CHA, ">", 'pulchra_chain.pdb' or die;

$oldres=-999;
$nchain=0;
$chain=();
@refchain='A' .. 'Z';

while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,27,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain];         
             $nchain++;
             }
           if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } else {print CHA $_,"\n"};
	   $oldres=$res;
           	}
close(CHA);
close(SID);

open OPT, ">", 'optimize.py' or die;
print OPT "\nfrom modeller import *";
print OPT "\nfrom modeller.scripts import complete_pdb";
print OPT "\nfrom modeller.optimizers import conjugate_gradients, molecular_dynamics, actions";
print OPT "\nenv = environ()";
print OPT "\nenv.io.atom_files_directory = ['../atom_files']";
print OPT "\nenv.edat.dynamic_sphere = True";
print OPT "\nenv.libs.topology.read(file='\$(LIB)/top_heav.lib')";
print OPT "\nenv.libs.parameters.read(file='\$(LIB)/par.lib')";
print OPT "\ncode = 'pulchra_chain.pdb'";
print OPT "\nmdl = complete_pdb(env, code)";
print OPT "\nmdl.write(file=code+'.ini')";
print OPT "\natmsel = selection(mdl)";
print OPT "\nmdl.restraints.make(atmsel, restraint_type='stereo', spline_on_site=False)";
print OPT "\nmdl.restraints.write(file=code+'.rsr')";
print OPT "\nmpdf = atmsel.energy()";
print OPT "\ncg = conjugate_gradients(output='REPORT')";
print OPT "\ntrcfil = file(code+'.D00000001', 'w')";
print OPT "\ncg.optimize(atmsel, max_iterations=5)";
print OPT "\nmpdf = atmsel.energy()";
print OPT "\nmdl.write(file='model.pdb')";
close(OPT);

system("$modeller optimize.py");
}

system("cat box.pdb model.pdb > protein_box.pdb");

if ($ff eq "GROMOS96-43a1"){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff gromos43a1 -water spc ");
} elsif ($ff eq "GROMOS96-43a2"){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff gromos43a2 -water spc ");
} elsif ($ff eq "GROMOS96-45a3"){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff gromos45a3 -water spc ");
} elsif ($ff eq "GROMOS96-53a5"){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff gromos53a5 -water spc ");
} elsif ($ff eq "GROMOS96-53a6"){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff gromos53a6 -water spc ");
} elsif ($ff =~ m/^OPLS/){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff oplsaa -water tip4p ");
} elsif ($ff =~ m/^CHARMM27/){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff charmm27 -water tip3p ");
} elsif ($ff =~ m/^CHARMM36/){
system("$gromacs/pdb2gmx -f protein_box.pdb -o p2g.pdb -p protein.top -ff charmm36 -water tip3p ");
}


system("$gromacs/grompp -maxwarn 10 -f em.mdp -o prot_em -c p2g.pdb -p protein.top");
system("$gromacs/mdrun_d -deffnm prot_em");
system("$gromacs/editconf -resnr 1 -f prot_em.gro -o atomistic-protein.pdb");

open (SID, "atomistic-protein.pdb")  || die "Energy minimisation of AT protein failed\n";

$oldres=-999;
$nchain=0;
$chain=();
@refchain='A' .. 'Z'; 
  
open CHA, ">", 'atomistic-protein-chain.pdb' or die;

while (<SID>) {
           $len=length($_);
           $str1=(substr($_,0,21));
	   $str2=(substr($_,27,($len-26)));
	   chomp;
	   ($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
           $res=$woe;
	   if (($res==$chain_num[$nchain]) && ($res ne $oldres) && ($woa eq "ATOM")) {
             $oldres=$res;
             $chain=$refchain[$nchain];         
             $nchain++;
             }
           if (length($res)==4) {
             print CHA $str1,"",$chain,"",$res,"",$str2;
             } elsif (length($res)==3) {
             	print CHA $str1,"",$chain," ",$res,"",$str2;
             	} elsif (length($res)==2) {
                    print CHA $str1,"",$chain,"  ",$res,"",$str2;
            		} elsif (length($res)==1) {
             		print CHA $str1,"",$chain,"   ",$res,"",$str2;
           		 } else {print CHA $_,"\n"};
	   $oldres=$res;
           	}
close(CHA);
close(SID);

open CHA, "<", 'atomistic-protein-chain.pdb' or die;
close(CHA);

####################COMBINE PROTEIN WITH LIPID###################

open (PRO, "protein.top")  || die "Something went wrong\n";
open(TOP, "> topol.top") || die "No Output Possible\n";
my @pro = <PRO>;
if ($ff eq "GROMOS96-43a1"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"gromos-lipids.itp\"\n\#include \"lipid-berger.itp\"\n/g for @pro;
} elsif ($ff eq "GROMOS96-43a2"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"gromos-lipids.itp\"\n\#include \"lipid-berger.itp\"\n/g for @pro;
} elsif ($ff eq "GROMOS96-45a3"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"gromos-lipids.itp\"\n\#include \"lipid-berger.itp\"\n/g for @pro;
} elsif ($ff eq "GROMOS96-53a5"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"gromos-lipids.itp\"\n\#include \"lipid-berger.itp\"\n/g for @pro;
} elsif ($ff eq "GROMOS96-53a6"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"gromos-lipids.itp\"\n\#include \"lipid-berger.itp\"\n/g for @pro;
} elsif ($ff eq "OPLSAA"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"oplsaa-lipids.itp\"\n\#include \"lipid-oplsaa.itp\"\n/g for @pro;
}elsif ($ff eq "OPLSUA"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"oplsua-lipids.itp\"\n\#include \"lipid-oplsua.itp\"\n/g for @pro;
}elsif ($ff eq "CHARMM27"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"charmm27-lipids.itp\"\n/g for @pro;
}elsif ($ff eq "CHARMM36"){
s/forcefield.itp\"/forcefield.itp\"\n\#include \"charmm36-lipids.itp\"\n/g for @pro;
}
print TOP @pro;

if ($lipidnum > 0) {

if ($method eq "align"){
if ($deletion eq "genbox"){
system("$gromacs/genbox -cp atomistic-protein.pdb -cs atomistic-lipid.pdb -o prot+lipid+box.pdb");
}

elsif ($deletion eq "g_membed"){
system("cat atomistic-protein.pdb atomistic-lipid.pdb | grep ATOM > prot+lipid.pdb");
system("cat box.pdb prot+lipid.pdb > prot-lipid-box.pdb");
system("cp topol.top membed.top");

open GMDP, ">", 'membed.mdp' or die;
print GMDP "\nintegrator      = md";
print GMDP "\ntinit           = 0.0";
print GMDP "\ndt              = 0.002";
print GMDP "\nnsteps          = 1000";
print GMDP "\nenergygrps      = Protein";
print GMDP "\nfreezegrps      = Protein";
print GMDP "\nfreezedim       = Y Y Y";
print GMDP "\nenergygrp_excl  = Protein Protein";
close(GMDP);

$popc = 0;
$pope = 0;
$popg = 0;
$pops = 0;
$dopc = 0;
$dppc = 0;
$dmpc = 0;
$dmpg = 0;
$dspe = 0;
$dhpc = 0;
$pip3 = 0;
$pip2 = 0;
$bog = 0;
$chol = 0;
$ddm = 0;

open (INP, "prot-lipid-box.pdb") || die "Cannot open prot-lipid-box.pdb\n";
$atom = 0;
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
        if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq C1)&&($wod eq POP))){                  
        $popc++;          
        }
        if (($ff =~ m/^CHARMM/)&&(($woc eq NC3)&&($wod eq POP))){   
        $popc++;                
        }
        if (($ff =~ m/^CHARMM/)&&(($woc eq NH3)&&($wod eq POP))){   
        $pope++;                
        }
	if (($ff =~ m/^CHARMM/)&&(($woc eq C1H)&&($wod eq POP))){   
        $popg++;                
        }
	if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq H1)&&($wod eq POP))){
	$pope++;
	}
	if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq H0)&&($wod eq POP))){
	$popg++;
	}
	if (($woc eq P11)&&($wod eq POP)){
	$pops++;
	}
	if (($woc eq C1)&&($wod eq DOP)){
	$dopc++;
	}
	if (($woc eq H1)&&($wod eq DSP)){
	$dspe++;
	}
	if (($woc eq C1)&&($wod eq DPP)){
	$dppc++;
	}
	if (($woc eq C1)&&($wod eq DMP)){
	$dmpc++;
	}
	if (($woc eq H0)&&($wod eq DMP)){
	$dmpg++;
	}
	if (($woc eq C1)&&($wod eq DHP)){
	$dhpc++;
	}
	if (($woc eq HBE)&&($wod eq PIP)){
	$pip2++;
	}
	if (($woc eq PBE)&&($wod eq PIP)){
	$pip3++;
	}
	if (($woc eq C10)&&($wod eq BOG)){
	$bog++;
	}
	if (($woc eq C10)&&($wod eq DDM)){
	$ddm++;
	}
	if (($woc eq C10)&&($wod eq CHO)){
	$chol++;
	}
}
close (INP);

open(OUT, ">> membed.top") || die "No Output Possible\n";

unless ($pip2 == 0){
print OUT "\nPIP2 $pip2";
}
unless ($pip3 == 0){
print OUT "\nPIP3 $pip3";
}
unless ($popc == 0){ 
print OUT "\nPOPC $popc";
}
unless ($popg == 0){ 
print OUT "\nPOPG $popg";
}
unless ($pope == 0){
print OUT "\nPOPE $pope";
}
unless ($pops == 0){
print OUT "\nPOPS $pops";
}
unless ($dopc == 0){
print OUT "\nDOPC $dopc";
}
unless ($dspe == 0){
print OUT "\nDSPE $dspe";
}
unless ($dppc == 0){ 
print OUT "\nDPPC $dppc";
}
unless ($dmpc == 0){
print OUT "\nDMPC $dmpc";
}
unless ($dmpg == 0){
print OUT "\nDMPG $dmpg";
}
unless ($dhpc == 0){
print OUT "\nDHPC $dhpc";
}
unless ($bog == 0){ 
print OUT "\nBOG $bog\n";
}
unless ($ddm == 0){ 
print OUT "\nDDM $ddm\n";
}
unless ($chol == 0){ 
print OUT "\nCHOL $chol\n";
}
close (OUT);

system("$gromacs/make_ndx -f prot-lipid-box.pdb -o membed.ndx  << EOD
del 2-30
rPOP|rPIP|rDHP|rDPP|rDMP|rDOP|rBOG|rCHO|rDDM
q
EOD");

system("$gromacs/grompp -f membed.mdp -p membed.top -c prot-lipid-box.pdb -o membed.tpr -maxwarn 5");
system("$gromacs/g_membed -f membed.tpr -p membed.top -c prot+lipid+box.pdb -xyinit 0.3 <<EOD
1
2");
}

elsif ($deletion eq "pymol"){

system("cat atomistic-protein.pdb atomistic-lipid.pdb | grep ATOM > prot-lipid.pdb");

open (PYMOL, "> lipid.pml") || die "Cannot open lipid.pml\n";
print PYMOL "set retain_order,1\n";
print PYMOL "set pdb_retain_ids,1\n";
print PYMOL "load prot-lipid.pdb\n";
print PYMOL "select deletion, bm. polymer\n";
print PYMOL "remove br. deletion\n";
print PYMOL "save atomistic-lipids-hole.pdb\n";
close (PYMOL);

system("$pymol -c lipid.pml");

system("cat atomistic-protein.pdb atomistic-lipids-hole.pdb | grep ATOM > prot+lipid.pdb");
system("cat box.pdb prot+lipid.pdb > prot+lipid+box.pdb");
	}

elsif ($deletion eq "none"){
system("cat atomistic-protein.pdb atomistic-lipid.pdb | grep ATOM > prot+lipid.pdb");
system("cat box.pdb prot+lipid.pdb > prot+lipid+box.pdb");
	}

}

elsif ($method eq "full"){
system("cat atomistic-protein.pdb atomistic-lipid.pdb | grep ATOM > prot+lipid.pdb");
system("cat box.pdb prot+lipid.pdb > prot+lipid+box.pdb");
}

$popc = 0;
$pope = 0;
$popg = 0;
$pops = 0;
$dopc = 0;
$dspe = 0;
$dppc = 0;
$dmpc = 0;
$dmpg = 0;
$dhpc = 0;
$pip3 = 0;
$pip2 = 0;
$bog = 0;
$chol = 0;
$ddm = 0;

open (INP, "prot+lipid+box.pdb") || die "Cannot open prot+lipid+box.pdb\n";
$atom = 0;
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
        if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq C1)&&($wod eq POP))){                  
        $popc++;          
        }
        if (($ff =~ m/^CHARMM/)&&(($woc eq NC3)&&($wod eq POP))){   
        $popc++;                
        }
        if (($ff =~ m/^CHARMM/)&&(($woc eq NH3)&&($wod eq POP))){   
        $pope++;                
        }
	if (($ff =~ m/^CHARMM/)&&(($woc eq C1H)&&($wod eq POP))){   
        $popg++;                
        }
	if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq H1)&&($wod eq POP))){
	$pope++;
	}
	if (($ff =~ m/^GROMOS/||$ff =~ m/^OPLS/)&&(($woc eq H0)&&($wod eq POP))){
	$popg++;
	}
	if (($woc eq P11)&&($wod eq POP)){
	$pops++;
	}
	if (($woc eq C1)&&($wod eq DOP)){
	$dopc++;
	}
	if (($woc eq H1)&&($wod eq DSP)){
	$dspe++;
	}
	if (($woc eq C1)&&($wod eq DPP)){
	$dppc++;
	}
	if (($woc eq C1)&&($wod eq DMP)){
	$dmpc++;
	}
	if (($woc eq H0)&&($wod eq DMP)){
	$dmpg++;
	}
	if (($woc eq C1)&&($wod eq DHP)){
	$dhpc++;
	}
	if (($woc eq HBE)&&($wod eq PIP)){
	$pip2++;
	}
	if (($woc eq PBE)&&($wod eq PIP)){
	$pip3++;
	}
	if (($woc eq C10)&&($wod eq BOG)){
	$bog++;
	}
	if (($woc eq C10)&&($wod eq DDM)){
	$ddm++;
	}
	if (($woc eq C10)&&($wod eq CHO)){
	$chol++;
	}
}
close (INP);
unless ($pip2 == 0){
print TOP "\nPIP2 $pip2";
}
unless ($pip3 == 0){
print TOP "\nPIP3 $pip3";
}
unless ($popc == 0){ 
print TOP "\nPOPC $popc";
}
unless ($popg == 0){ 
print TOP "\nPOPG $popg";
}
unless ($pope == 0){
print TOP "\nPOPE $pope";
}
unless ($pops == 0){
print TOP "\nPOPS $pops";
}
unless ($dopc == 0){
print TOP "\nDOPC $dopc";
}
unless ($dspe == 0){
print TOP "\nDSPE $dspe";
}
unless ($dppc == 0){ 
print TOP "\nDPPC $dppc";
}
unless ($dmpc == 0){
print TOP "\nDMPC $dmpc";
}
unless ($dmpg == 0){
print TOP "\nDMPG $dmpg";
}
unless ($dhpc == 0){
print TOP "\nDHPC $dhpc";
}
unless ($bog == 0){ 
print TOP "\nBOG $bog\n";
}
unless ($ddm == 0){ 
print TOP "\nDDM $ddm\n";
}
unless ($chol == 0){ 
print TOP "\nCHOL $chol\n";
}
close (TOP);
close (LIP);

if ($deletion eq "none"){
system("$gromacs/grompp -maxwarn 10 -f em-cg.mdp -o prot_lipid_em -c prot+lipid+box.pdb -p topol.top");
}
else{
system("$gromacs/grompp -maxwarn 10 -f em.mdp -o prot_lipid_em -c prot+lipid+box.pdb -p topol.top");
}
system("$gromacs/mdrun_d -deffnm prot_lipid_em");
system("$gromacs/editconf -resnr 1 -f prot_lipid_em.gro -o atomistic-protein-lipid.pdb");

open (ATM, "atomistic-protein-lipid.pdb") || die "Energy minimisation of AT protein-lipid complex failed\n";
close(ATM);

`cp atomistic-protein-lipid.pdb temp.pdb`;
	}
else{
`cp atomistic-protein.pdb temp.pdb`;
	}
}
else{
`cp atomistic-lipid.pdb temp.pdb`;
open (LIP, "lipid.top")  || die "Something went wrong\n";
open(TOP, "> topol.top") || die "No Output Possible\n";
my @lip = <LIP>;
if ($ff =~ m/^GROMOS/){
s/\[ system \]/\n\#include "spc.itp\"\n\#include "ions.itp\"\n\[ system \]\n/g for @lip;
}
if ($ff =~ m/^OPLS/){
s/\[ system \]/\n\#include "tip4p.itp\"\n\#include "ions.itp\"\n\[ system \]\n/g for @lip;
}
if ($ff =~ m/^CHARMM27/){
s/\[ system \]/\n\#include "charmm27.ff\/tip3p.itp\"\n\#include "charmm27.ff\/ions.itp\"\n\[ system \]\n/g for @lip;
}
if ($ff =~ m/^CHARMM36/){
s/\[ system \]/\n\#include "charmm36.ff\/tip3p.itp\"\n\#include "charmm36.ff\/ions.itp\"\n\[ system \]\n/g for @lip;
}
print TOP @lip;
close (TOP);
close (LIP);
}

#############################ADD WATER###############################

if ($delete_waters eq "genbox"){
open VDW, ">", 'vdwradii.dat' or die;
print VDW "\nAAA  C     0.15";
print VDW "\n???  F     0.12";
print VDW "\n???  H     0.04";
print VDW "\n???  N     0.110";
print VDW "\n???  O     0.105";
print VDW "\n???  S     0.16";
print VDW "\nCHO  C2    0.15";
print VDW "\nPOP  CA1   0.75";
print VDW "\nPOP  CA2   0.75";
print VDW "\nDSP  CA1   0.75";
print VDW "\nDSP  CA2   0.75";
print VDW "\nDSP  CA3   0.75";
print VDW "\nDSP  CA4   0.75";
print VDW "\nPOP  C54   0.75";
print VDW "\nPOP  C55   0.75";
print VDW "\nCHO  C27   0.75";
print VDW "\nCHO  C28   0.75";
print VDW "\nCHO  C29   0.75";
print VDW "\nDOP  CA1   0.75";
print VDW "\nDOP  CA2   0.75";
print VDW "\nDOP  CA3   0.75";
print VDW "\nDOP  CA4   0.75";
print VDW "\nDPP  C30   0.75";
print VDW "\nDPP  C31   0.75";
print VDW "\nDPP  C49   0.75";
print VDW "\nDPP  C50   0.75";
print VDW "\nDMP  C28   0.75";
print VDW "\nDMP  C29   0.75";
print VDW "\nDMP  C45   0.75";
print VDW "\nDMP  C46   0.75";
print VDW "\nPIP  CAA   0.75";
print VDW "\nPIP  CAB   0.75";
print VDW "\nPIP  CCO   0.75";
print VDW "\nPIP  CCN   0.75";
close(VDW);
}

if ($ff =~ m/^GROMOS/){
system("$gromacs/genbox -cs spc216.gro -cp temp.pdb -o solvated.pdb");
}
if ($ff =~ m/^OPLS/){
system("$gromacs/genbox -cs tip4p.gro -cp temp.pdb -o solvated.pdb");
}
if ($ff =~ m/^CHARMM/){
system("$gromacs/genbox -cs spc216.gro -cp temp.pdb -o solvated.pdb");
}
system("rm vdwradii.dat");

if ($delete_waters eq "waterkiller"){

$solvated = "solvated.pdb";
$outfile = "sol_kill.pdb";
$outfile2 = "prot+lipid+sol.pdb";

$numWaterRemoved = 0;			

$protMinX = 1000;
$protMinY = 1000;
$protMinZ = 1000;
$protMaxX = 0;
$protMaxY = 0;
$protMaxZ = 0;

open(IN, $solvated);
	until(eof IN){
		$linebuffer = <IN>;
		@tokens = split ' ', $linebuffer;
		if(($tokens[0] eq "ATOM")&&(($tokens[2] eq "C20")||($tokens[2] eq "C25"))){
			$atomZ = $tokens[7];
				if($atomZ < $protMinZ){
					$protMinZ = $atomZ;
				}
				if($atomZ > $protMaxZ){
					$protMaxZ = $atomZ;
				}
			}
		}


open(OUT, ">./$outfile");

$protMinX += 10;
$protMinY += 10;
$protMaxX -= 10;
$protMaxY -= 10;
			
close(IN);
open(IN, $solvated);

#now kill the waters
until(eof IN){
	$linebuffer = <IN>;
	@tokens = split ' ', $linebuffer;
	
	$atomX = $tokens[5];
	$atomY = $tokens[6];
	$atomZ = $tokens[7];

	if(($tokens[0] eq "ATOM") && ($tokens[3] eq "SOL") && 
	   ($tokens[2] eq "OW")  && ($atomZ > $protMinZ) && ($atomZ < $protMaxZ)){
		##ITS a water molecule in the kill z range, but is it in the protein box?
		if(($atomX > $protMinX) && ($atomX < $protMaxX) && ($atomY > $protMinY) && ($atomY < $protMaxY)){
			print "Water #$tokens[4] in protein and saved\n";
			print OUT $linebuffer;
		}
		else{
			$linebuffer = <IN>;
			$linebuffer = <IN>;
			if ($ff =~ m/^OPLS/){
			$linebuffer = <IN>;
			}			
			$numWaterRemoved++;
		}
	}
	else{
		print OUT $linebuffer;
	}
}

print "$numWaterRemoved Waters were removed\n";

close(IN);
close(OUT);

system("$gromacs/editconf -resnr 1 -f $outfile -o $outfile2");
}

else {
system("$gromacs/editconf -resnr 1 -f solvated.pdb -o prot+lipid+sol.pdb");
	}

open (SOL, "prot+lipid+sol.pdb") || die "Cannot open prot+lipid+sol.pdb\n";

$sol = 0;

while (<SOL>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	if ($woc eq OW){
	$sol++;
	}
}
close (SOL);

open(TOP, "topol.top") || die "Something went wrong\n";
open(SOLTOP, "> sol.top") || die "No Output Possible\n";
	my @top = <TOP>;
	if ($ff =~ m/^GROMOS/){
	s/\#include \"spc.itp\"/\#ifdef FLEX_SPC\n\#include \"flexspc.itp\"\n\#else\n#include \"spc.itp"\n\#endif/g for @top;
	}
	if ($ff =~ m/^OPLS/){
	s/\#include \"spc.itp\"/\#include \"tip4p.itp\"/g for @top;
	}
	if ($ff =~ m/^CHARMM/){
	s/\#include \"spc.itp\"/\#include \"tip3p.itp\"/g for @top;
	}
	print SOLTOP @top;
        printf (SOLTOP "\nSOL %d\n", $sol );
close (TOP);
close (SOLTOP);

open(TOP, "topol.top") || die "Something went wrong\n";
open(OUT, "> topol1.top") || die "No Output Possible\n";
	my @top = <TOP>;
	if ($ff =~ m/^GROMOS/){
	s/\#include \"spc.itp\"/\#ifdef FLEX_SPC\n\#include \"flexspc.itp\"\n\#else\n#include \"spc.itp"\n\#endif/g for @top;
	}
	if ($ff =~ m/^OPLS/){
	s/\#include \"spc.itp\"/\#include \"tip4p.itp\"/g for @top;
	}
	if ($ff =~ m/^CHARMM27/){
	s/\#include \"spc.itp\"/\#include \"charmm27.ff\/tip3p.itp\"/g for @top;
	}
	if ($ff =~ m/^CHARMM36/){
	s/\#include \"spc.itp\"/\#include \"charmm36.ff\/tip3p.itp\"/g for @top;
	}
	print OUT @top;
close (TOP);


##############################ADD IONS###############################

`$gromacs/grompp -maxwarn 10 -f em.mdp -o em -c prot+lipid+sol.pdb -p sol.top`;

system("$gromacs/make_ndx -f prot+lipid+sol.pdb -o solvent.ndx << EOD
del 0-25
r SOL
q
EOD");

system("$gromacs/genion -s em.tpr -o prot+lipid+sol+ion.pdb -neutral -conc 0.15 -n solvent.ndx << EOD
0
EOD");

my $na = 0;
my $sol = 0;
my $cl = 0;

open (INP, "prot+lipid+sol+ion.pdb") || die "Cannot open prot+lipid+box.pdb\n";
$atom = 0;
while (<INP>){
	chomp;
	($woa, $wob, $woc, $wod, $woe, $wof, $wog) = split;
	if (($woc eq NA)){
	$na++;
	}
	if (($woc eq CL)){
	$cl++;
	}
	if (($woc eq OW)){
	$sol++;
	}
}
close (INP);

print OUT "\nSOL $sol";

if ($ff =~ m/^OPLS/){
print OUT "\nNA $na";
print OUT "\nCL $cl";
}
if ($ff =~ m/^GROMOS/){
print OUT "\nNA $na";
print OUT "\nCL $cl";
}
close (OUT);

open(EMV, "em.mdp") || die "Something went wrong\n";
open(EMS, "> em-system.mdp") || die "No Output Possible\n";

while (<EMV>) {
if ($_ !~ /epsilon_r/) {
	print EMS $_;
	}
}
print EMS "\ndefine                   = -DFLEX_SPC\n";
close (EMS);
close (EMV);

system("$gromacs/grompp -maxwarn 10 -f em-system.mdp -o system_em -c prot+lipid+sol+ion.pdb -p topol1.top");
system("$gromacs/mdrun_d -deffnm system_em");
system("$gromacs/editconf -resnr 1 -f system_em.gro -o atomistic-system.pdb");

mkdir('itp_files');
`cp *.itp itp_files`;
`mv topol1.top topol`;

# Cleaning up...
`rm -rf ddm.pdb dspe.pdb order-* cg.top em.tpr martini* ff_v1.4_x.itp chol.pdb Res_CG/ fit_*/ Sidechains/ *.ndx *v*.itp z *.log wat.pdb order.pdb box.pdb ion_atoms.pdb lipid.pdb p2g.pdb em_inbox* *_em* *bilayer* '#'* *_chain* NA* CL* pip* pop* dpp* bog* temp* sol* *.py  dhp* dmp* *label* mdout.mdp model.pdb chain* dop* pro*.pdb inbox.pdb coord.xvg charge.txt`;
`mv topol topol.top`;
`mv itp_files/* .`;
`rm -rf itp_files #* *membed* *.cpt nohydrogens.pdb order-center* em.tpr ddm* ener* chol* cg.top lipid.top traj.trr`;

open (ATSYS, "atomistic-system.pdb") || die "Something went wrong\n";
close (ATSYS);
print "\n";
print "\n Atomistic conversion complete! see:";
print "\n atomistic-system.pdb and other files with the atomistic- prefix";
print "\n";
